@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #ifndef DECO_H
 #define DECO_H
 
@@ -5,11 +6,8 @@
 extern "C" {
 #endif
 
-extern double tolerated_by_tissue[];
 extern double buehlmann_N2_t_halflife[];
-extern double tissue_inertgas_saturation[16];
-extern double buehlmann_inertgas_a[16], buehlmann_inertgas_b[16];
-extern double gf_low_pressure_this_dive;
+extern struct deco_state *deco_state;
 
 extern int deco_allowed_depth(double tissues_tolerance, double surface_pressure, struct dive *dive, bool smooth);
 
