@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #ifndef DIVE_H
 #define DIVE_H
 
@@ -138,7 +139,8 @@ extern int units_to_sac(double volume);
 extern int gas_volume(cylinder_t *cyl, pressure_t p);
 extern double gas_compressibility_factor(struct gasmix *gas, double bar);
 extern double isothermal_pressure(struct gasmix *gas, double p1, int volume1, int volume2);
-
+extern double gas_density(struct gasmix *gas, int pressure);
+extern int same_gasmix(struct gasmix *a, struct gasmix *b);
 
 static inline int get_o2(const struct gasmix *mix)
 {
@@ -181,6 +183,7 @@ static inline int interpolate(int a, int b, int part, int whole)
 void get_gas_string(const struct gasmix *gasmix, char *text, int len);
 const char *gasname(const struct gasmix *gasmix);
 
+#define MAX_SENSORS 2
 struct sample                         // BASE TYPE BYTES  UNITS    RANGE      DESCRIPTION
 {                                     // --------- -----  -----    -----      -----------
 	duration_t time;               // uint32_t   4  seconds  (0-68 yrs)   elapsed dive time up to this sample
@@ -191,12 +194,11 @@ struct sample                         // BASE TYPE BYTES  UNITS    RANGE      DE
 	depth_t depth;                 // int32_t    4    mm     (0-2000 km)  dive depth of this sample
 	depth_t stopdepth;             // int32_t    4    mm     (0-2000 km)  depth of next deco stop
 	temperature_t temperature;     // int32_t    4  mdegrK   (0-2 MdegK)  ambient temperature
-	pressure_t cylinderpressure;   // int32_t    4    mbar   (0-2 Mbar)   main cylinder pressure
-	pressure_t o2cylinderpressure; // int32_t    4    mbar   (0-2 Mbar)   CCR o2 cylinder pressure (rebreather)
+	pressure_t pressure[MAX_SENSORS]; // int32_t    4    mbar   (0-2 Mbar)   cylinder pressures (main and CCR o2)
 	o2pressure_t setpoint;         // uint16_t   2    mbar   (0-65 bar)   O2 partial pressure (will be setpoint)
 	o2pressure_t o2sensor[3];      // uint16_t   6    mbar   (0-65 bar)   Up to 3 PO2 sensor values (rebreather)
 	bearing_t bearing;             // int16_t    2  degrees  (-32k to 32k deg) compass bearing
-	uint8_t sensor;                // uint8_t    1  sensorID (0-255)      ID of cylinder pressure sensor
+	uint8_t sensor[MAX_SENSORS];   // uint8_t    1  sensorID (0-255)      ID of cylinder pressure sensor
 	uint8_t cns;                   // uint8_t    1     %     (0-255 %)    cns% accumulated
 	uint8_t heartbeat;             // uint8_t    1  beats/m  (0-255)      heart rate measurement
 	volume_t sac;                  //            4  ml/min                predefined SAC
@@ -293,6 +295,7 @@ struct divecomputer {
 
 #define MAX_CYLINDERS (20)
 #define MAX_WEIGHTSYSTEMS (6)
+#define MAX_TANK_INFO (100)
 #define W_IDX_PRIMARY 0
 #define W_IDX_SECONDARY 1
 
@@ -349,7 +352,6 @@ struct dive {
 	struct divecomputer dc;
 	int id; // unique ID for this dive
 	struct picture *picture_list;
-	int oxygen_cylinder_index, diluent_cylinder_index; // CCR dive cylinder indices
 	unsigned char git_id[20];
 };
 
@@ -365,7 +367,8 @@ static inline bool dive_cache_is_valid(const struct dive *dive)
 }
 
 extern int get_cylinder_idx_by_use(struct dive *dive, enum cylinderuse cylinder_use_type);
-extern void dc_cylinder_renumber(struct dive *dive, struct divecomputer *dc, int mapping[]);
+extern void cylinder_renumber(struct dive *dive, int mapping[]);
+extern int same_gasmix_cylinder(cylinder_t *cyl, int cylid, struct dive *dive, bool check_unused);
 
 /* when selectively copying dive information, which parts should be copied? */
 struct dive_components {
@@ -546,6 +549,7 @@ extern int selected_dive;
 extern unsigned int dc_number;
 #define current_dive (get_dive(selected_dive))
 #define current_dc (get_dive_dc(current_dive, dc_number))
+#define displayed_dc (get_dive_dc(&displayed_dive, dc_number))
 
 static inline struct dive *get_dive(int nr)
 {
@@ -568,6 +572,17 @@ static inline struct dive_site *get_dive_site_for_dive(struct dive *dive)
 	return NULL;
 }
 
+static inline const char *get_dive_country(struct dive *dive)
+{
+	struct dive_site *ds = get_dive_site_by_uuid(dive->dive_site_uuid);
+	if (ds) {
+		int idx = taxonomy_index_for_category(&ds->taxonomy, TC_COUNTRY);
+		if (idx >= 0)
+			return ds->taxonomy.category[idx].value;
+	}
+	return NULL;
+}
+
 static inline char *get_dive_location(struct dive *dive)
 {
 	struct dive_site *ds = get_dive_site_by_uuid(dive->dive_site_uuid);
@@ -593,7 +608,10 @@ static inline unsigned int number_of_computers(struct dive *dive)
 
 static inline struct divecomputer *get_dive_dc(struct dive *dive, int nr)
 {
-	struct divecomputer *dc = &dive->dc;
+	struct divecomputer *dc;
+	if (!dive)
+		return NULL;
+	dc = &dive->dc;
 
 	while (nr-- > 0) {
 		dc = dc->next;
@@ -703,6 +721,7 @@ extern int parse_dlf_buffer(unsigned char *buffer, size_t size);
 
 extern int parse_file(const char *filename);
 extern int parse_csv_file(const char *filename, char **params, int pnr, const char *csvtemplate);
+extern int parse_seabear_log(const char *filename);
 extern int parse_seabear_csv_file(const char *filename, char **params, int pnr, const char *csvtemplate);
 extern int parse_txt_file(const char *filename, const char *csv);
 extern int parse_manual_file(const char *filename, char **params, int pnr);
@@ -756,6 +775,8 @@ extern void clear_table(struct dive_table *table);
 
 extern struct sample *prepare_sample(struct divecomputer *dc);
 extern void finish_sample(struct divecomputer *dc);
+extern void add_sample_pressure(struct sample *sample, int sensor, int mbar);
+extern int legacy_format_o2pressures(struct dive *dive, struct divecomputer *dc);
 
 extern bool has_hr_data(struct divecomputer *dc);
 
@@ -830,13 +851,43 @@ extern void subsurface_command_line_exit(int *, char ***);
 
 #define FRACTION(n, x) ((unsigned)(n) / (x)), ((unsigned)(n) % (x))
 
+#define DECOTIMESTEP 60 /* seconds. Unit of deco stop times */
+
+struct deco_state {
+	double tissue_n2_sat[16];
+	double tissue_he_sat[16];
+	double tolerated_by_tissue[16];
+	double tissue_inertgas_saturation[16];
+	double buehlmann_inertgas_a[16];
+	double buehlmann_inertgas_b[16];
+
+	double max_n2_crushing_pressure[16];
+	double max_he_crushing_pressure[16];
+
+	double crushing_onset_tension[16];            // total inert gas tension in the t* moment
+	double n2_regen_radius[16];                   // rs
+	double he_regen_radius[16];
+	double max_ambient_pressure;                  // last moment we were descending
+
+	double bottom_n2_gradient[16];
+	double bottom_he_gradient[16];
+
+	double initial_n2_gradient[16];
+	double initial_he_gradient[16];
+	pressure_t first_ceiling_pressure;
+	pressure_t max_bottom_ceiling_pressure;
+	int ci_pointing_to_guiding_tissue;
+	double gf_low_pressure_this_dive;
+
+};
+
 extern void add_segment(double pressure, const struct gasmix *gasmix, int period_in_seconds, int setpoint, const struct dive *dive, int sac);
 extern void clear_deco(double surface_pressure);
 extern void dump_tissues(void);
-extern void set_gf(short gflow, short gfhigh, bool gf_low_at_maxdepth);
+extern void set_gf(short gflow, short gfhigh);
 extern void set_vpmb_conservatism(short conservatism);
-extern void cache_deco_state(char **datap);
-extern void restore_deco_state(char *data);
+extern void cache_deco_state(struct deco_state **datap);
+extern void restore_deco_state(struct deco_state *data, bool keep_vpmb_state);
 extern void nuclear_regeneration(double time);
 extern void vpmb_start_gradient();
 extern void vpmb_next_gradient(double deco_time, double surface_pressure);
@@ -864,7 +915,7 @@ struct diveplan {
 	short vpmb_conservatism;
 	struct divedatapoint *dp;
 	int eff_gflow, eff_gfhigh;
-	unsigned int surface_interval;
+	int surface_interval;
 };
 
 struct divedatapoint *plan_add_segment(struct diveplan *diveplan, int duration, int depth, int cylinderid, int po2, bool entered);
@@ -872,15 +923,37 @@ struct divedatapoint *create_dp(int time_incr, int depth, int cylinderid, int po
 #if DEBUG_PLAN
 void dump_plan(struct diveplan *diveplan);
 #endif
-bool plan(struct diveplan *diveplan, char **cached_datap, bool is_planner, bool show_disclaimer);
+struct decostop {
+	int depth;
+	int time;
+};
+bool plan(struct diveplan *diveplan, struct dive *dive, int timestep, struct decostop *decostoptable, struct deco_state **cached_datap, bool is_planner, bool show_disclaimer);
 void calc_crushing_pressure(double pressure);
 void vpmb_start_gradient();
 void clear_vpmb_state();
+void printdecotable(struct decostop *table);
 
 void delete_single_dive(int idx);
 
 struct event *get_next_event(struct event *event, const char *name);
 
+static inline struct gasmix *get_gasmix(struct dive *dive, struct divecomputer *dc, int time, struct event **evp, struct gasmix *gasmix)
+{
+	struct event *ev = *evp;
+
+	if (!gasmix) {
+		int cyl = explicit_first_cylinder(dive, dc);
+		gasmix = &dive->cylinder[cyl].gasmix;
+		ev = dc ? dc->events : NULL;
+	}
+	while (ev && ev->time.seconds < (unsigned int)time) {
+		gasmix = get_gasmix_from_event(dive, ev);
+		ev = get_next_event(ev->next, "gaschange");
+	}
+	*evp = ev;
+	return gasmix;
+}
+
 
 /* these structs holds the information that
  * describes the cylinders / weight systems.
@@ -893,7 +966,7 @@ struct tank_info_t {
 	const char *name;
 	int cuft, ml, psi, bar;
 };
-extern struct tank_info_t tank_info[100];
+extern struct tank_info_t tank_info[MAX_TANK_INFO];
 
 struct ws_info_t {
 	const char *name;
@@ -941,6 +1014,8 @@ extern volume_t string_to_volume(const char *str, pressure_t workp);
 extern fraction_t string_to_fraction(const char *str);
 extern void average_max_depth(struct diveplan *dive, int *avg_depth, int *max_depth);
 
+extern struct dive_table downloadTable;
+
 #include "pref.h"
 
 #endif // DIVE_H
