@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #ifndef PREF_H
 #define PREF_H
 
@@ -125,6 +126,7 @@ struct preferences {
 	bool display_runtime;
 	bool display_duration;
 	bool display_transitions;
+	bool display_variations;
 	bool safetystop;
 	bool switch_at_req_stop;
 	int reserve_gas;
@@ -173,7 +175,8 @@ enum cloud_status {
 	CS_UNKNOWN,
 	CS_INCORRECT_USER_PASSWD,
 	CS_NEED_TO_VERIFY,
-	CS_VERIFIED
+	CS_VERIFIED,
+	CS_NOCLOUD
 };
 
 extern struct preferences prefs, default_prefs, git_prefs;
