@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #ifndef QTHELPERFROMC_H
 #define QTHELPERFROMC_H
 
@@ -19,5 +20,10 @@ char *hashfile_name_string();
 char *picturedir_string();
 const char *subsurface_user_agent();
 enum deco_mode decoMode();
+int parse_seabear_header(const char *filename, char **params, int pnr);
+extern const char *get_current_date();
+enum inertgas {N2, HE};
+double cache_value(int tissue, int timestep, enum inertgas gas);
+void cache_insert(int tissue, int timestep, enum inertgas gas, double value);
 
 #endif // QTHELPERFROMC_H
