@@ -1,7 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0
 /* statistics.c
  *
  * core logic for the Info & Stats page -
- * char *get_time_string(int seconds, int maxdays);
  * char *get_minutes(int seconds);
  * void process_all_dives(struct dive *dive, struct dive **prev_dive);
  * void get_selected_dives_text(char *buffer, int size);
@@ -235,27 +235,6 @@ void process_selected_dives(void)
 	stats_selection.selection_size = nr;
 }
 
-char *get_time_string_s(int seconds, int maxdays, bool freediving)
-{
-	static char buf[80];
-	if (maxdays && seconds > 3600 * 24 * maxdays) {
-		snprintf(buf, sizeof(buf), translate("gettextFromC", "more than %d days"), maxdays);
-	} else {
-		int days = seconds / 3600 / 24;
-		int hours = (seconds - days * 3600 * 24) / 3600;
-		int minutes = (seconds - days * 3600 * 24 - hours * 3600) / 60;
-		int secs = (seconds - days * 3600 * 24 - hours * 3600 - minutes*60);
-		if (days > 0)
-			snprintf(buf, sizeof(buf), translate("gettextFromC", "%dd %dh %dmin"), days, hours, minutes);
-		else
-			if (freediving && seconds < 3600)
-				snprintf(buf, sizeof(buf), translate("gettextFromC", "%dmin %dsecs"), minutes, secs);
-			else
-				snprintf(buf, sizeof(buf), translate("gettextFromC", "%dh %dmin"), hours, minutes);
-	}
-	return buf;
-}
-
 /* this gets called when at least two but not all dives are selected */
 static void get_ranges(char *buffer, int size)
 {
@@ -326,7 +305,8 @@ void get_selected_dives_text(char *buffer, size_t size)
 
 #define SOME_GAS 5000 // 5bar drop in cylinder pressure makes cylinder used
 
-bool has_gaschange_event(struct dive *dive, struct divecomputer *dc, int idx) {
+bool has_gaschange_event(struct dive *dive, struct divecomputer *dc, int idx)
+{
 	bool first_gas_explicit = false;
 	struct event *event = get_next_event(dc->events, "gaschange");
 	while (event) {
@@ -337,8 +317,12 @@ bool has_gaschange_event(struct dive *dive, struct divecomputer *dc, int idx) {
 			return true;
 		event = get_next_event(event->next, "gaschange");
 	}
-	if (dc->divemode == CCR && (idx == dive->diluent_cylinder_index || idx == dive->oxygen_cylinder_index))
-		return true;
+	if (dc->divemode == CCR) {
+		if (idx == get_cylinder_idx_by_use(dive, DILUENT))
+			return true;
+		if (idx == get_cylinder_idx_by_use(dive, OXYGEN))
+			return true;
+	}
 	return !first_gas_explicit && idx == 0;
 }
 
@@ -350,6 +334,10 @@ bool is_cylinder_used(struct dive *dive, int idx)
 
 	if ((dive->cylinder[idx].start.mbar - dive->cylinder[idx].end.mbar) > SOME_GAS)
 		return true;
+
+	if ((dive->cylinder[idx].sample_start.mbar - dive->cylinder[idx].sample_end.mbar) > SOME_GAS)
+		return true;
+
 	for_each_dc(dive, dc) {
 		if (has_gaschange_event(dive, dc, idx))
 			return true;
@@ -360,24 +348,11 @@ bool is_cylinder_used(struct dive *dive, int idx)
 void get_gas_used(struct dive *dive, volume_t gases[MAX_CYLINDERS])
 {
 	int idx;
-	struct divecomputer *dc;
-	bool used;
 
 	for (idx = 0; idx < MAX_CYLINDERS; idx++) {
-		used = false;
 		cylinder_t *cyl = &dive->cylinder[idx];
 		pressure_t start, end;
 
-		for_each_dc(dive, dc) {
-			if (same_string(dc->model, "planned dive"))
-				continue;
-			if (has_gaschange_event(dive, dc, idx))
-				used = true;
-		}
-
-		if (!used)
-			continue;
-
 		start = cyl->start.mbar ? cyl->start : cyl->sample_start;
 		end = cyl->end.mbar ? cyl->end : cyl->sample_end;
 		if (end.mbar && start.mbar > end.mbar)
