@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #include "SettingsObjectWrapper.h"
 #include <QSettings>
 #include <QApplication>
@@ -395,11 +396,6 @@ bool TechnicalDetailsSettings::showSac() const
 	return prefs.show_sac;
 }
 
-bool TechnicalDetailsSettings::gfLowAtMaxDepth() const
-{
-	return prefs.gf_low_at_maxdepth;
-}
-
 bool TechnicalDetailsSettings::displayUnusedTanks() const
 {
 	return prefs.display_unused_tanks;
@@ -560,7 +556,7 @@ void TechnicalDetailsSettings::setGflow(int value)
 	s.beginGroup(group);
 	s.setValue("gflow", value);
 	prefs.gflow = value;
-	set_gf(prefs.gflow, prefs.gfhigh, prefs.gf_low_at_maxdepth);
+	set_gf(prefs.gflow, prefs.gfhigh);
 	emit gflowChanged(value);
 }
 
@@ -573,7 +569,7 @@ void TechnicalDetailsSettings::setGfhigh(int value)
 	s.beginGroup(group);
 	s.setValue("gfhigh", value);
 	prefs.gfhigh = value;
-	set_gf(prefs.gflow, prefs.gfhigh, prefs.gf_low_at_maxdepth);
+	set_gf(prefs.gflow, prefs.gfhigh);
 	emit gfhighChanged(value);
 }
 
@@ -683,18 +679,6 @@ void TechnicalDetailsSettings::setShowSac(bool value)
 	emit showSacChanged(value);
 }
 
-void TechnicalDetailsSettings::setGfLowAtMaxDepth(bool value)
-{
-	if (value == prefs.gf_low_at_maxdepth)
-		return;
-	QSettings s;
-	s.beginGroup(group);
-	s.setValue("gf_low_at_maxdepth", value);
-	prefs.gf_low_at_maxdepth = value;
-	set_gf(prefs.gflow, prefs.gfhigh, prefs.gf_low_at_maxdepth);
-	emit gfLowAtMaxDepthChanged(value);
-}
-
 void TechnicalDetailsSettings::setDisplayUnusedTanks(bool value)
 {
 	if (value == prefs.display_unused_tanks)
@@ -1213,6 +1197,11 @@ bool DivePlannerSettings::displayTransitions() const
 	return prefs.display_transitions;
 }
 
+bool DivePlannerSettings::displayVariations() const
+{
+	return prefs.display_variations;
+}
+
 bool DivePlannerSettings::doo2breaks() const
 {
 	return prefs.doo2breaks;
@@ -1367,6 +1356,18 @@ void DivePlannerSettings::setDisplayTransitions(bool value)
 	emit displayTransitionsChanged(value);
 }
 
+void DivePlannerSettings::setDisplayVariations(bool value)
+{
+	if (value == prefs.display_variations)
+		return;
+
+	QSettings s;
+	s.beginGroup(group);
+	s.setValue("display_variations", value);
+	prefs.display_variations = value;
+	emit displayVariationsChanged(value);
+}
+
 void DivePlannerSettings::setDoo2breaks(bool value)
 {
 	if (value == prefs.doo2breaks)
@@ -1625,6 +1626,16 @@ int UnitsSettings::verticalSpeedTime() const
 	return prefs.units.vertical_speed_time;
 }
 
+int UnitsSettings::durationUnits() const
+{
+	return prefs.units.duration_units;
+}
+
+int UnitsSettings::showUnitsTable() const
+{
+	return prefs.units.show_units_table;
+}
+
 QString UnitsSettings::unitSystem() const
 {
 	return prefs.unit_system == METRIC ? QStringLiteral("metric")
@@ -1704,6 +1715,28 @@ void UnitsSettings::setVerticalSpeedTime(int value)
 	emit verticalSpeedTimeChanged(value);
 }
 
+void UnitsSettings::setDurationUnits(int value)
+{
+	if (value == prefs.units.duration_units)
+		return;
+	QSettings s;
+	s.beginGroup(group);
+	s.setValue("duration_units", value);
+	prefs.units.duration_units = (units::DURATION) value;
+	emit durationUnitChanged(value);
+}
+
+void UnitsSettings::setShowUnitsTable(int value)
+{
+	if (value == prefs.units.show_units_table)
+		return;
+	QSettings s;
+	s.beginGroup(group);
+	s.setValue("show_units_table", value);
+	prefs.units.show_units_table = value;
+	emit showUnitsTableChanged(value);
+}
+
 void UnitsSettings::setCoordinatesTraditional(bool value)
 {
 	if (value == prefs.coordinates_traditional)
@@ -2179,6 +2212,8 @@ void SettingsObjectWrapper::load()
 		GET_UNIT("weight", weight, units::LBS, units::KG);
 	}
 	GET_UNIT("vertical_speed_time", vertical_speed_time, units::MINUTES, units::SECONDS);
+	GET_UNIT3("duration_units", duration_units, units::MIXED, units::ALWAYS_HOURS, units::DURATION);
+	GET_UNIT_INT("show_units_table", show_units_table);
 	GET_BOOL("coordinates", coordinates_traditional);
 	s.endGroup();
 	s.beginGroup("TecDetails");
@@ -2209,7 +2244,7 @@ void SettingsObjectWrapper::load()
 	GET_BOOL("show_ccr_setpoint",show_ccr_setpoint);
 	GET_BOOL("show_ccr_sensors",show_ccr_sensors);
 	GET_BOOL("zoomed_plot", zoomed_plot);
-	set_gf(prefs.gflow, prefs.gfhigh, prefs.gf_low_at_maxdepth);
+	set_gf(prefs.gflow, prefs.gfhigh);
 	set_vpmb_conservatism(prefs.vpmb_conservatism);
 	GET_BOOL("show_sac", show_sac);
 	GET_BOOL("display_unused_tanks", display_unused_tanks);
@@ -2321,6 +2356,7 @@ void SettingsObjectWrapper::load()
 	GET_BOOL("display_duration", display_duration);
 	GET_BOOL("display_runtime", display_runtime);
 	GET_BOOL("display_transitions", display_transitions);
+	GET_BOOL("display_variations", display_variations);
 	GET_BOOL("safetystop", safetystop);
 	GET_BOOL("doo2breaks", doo2breaks);
 	GET_BOOL("switch_at_req_stop",switch_at_req_stop);
@@ -2379,6 +2415,7 @@ void SettingsObjectWrapper::sync()
 	s.setValue("display_duration", prefs.display_duration);
 	s.setValue("display_runtime", prefs.display_runtime);
 	s.setValue("display_transitions", prefs.display_transitions);
+	s.setValue("display_variations", prefs.display_variations);
 	s.setValue("safetystop", prefs.safetystop);
 	s.setValue("reserve_gas", prefs.reserve_gas);
 	s.setValue("ascrate75", prefs.ascrate75);
