@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #ifndef SETTINGSOBJECTWRAPPER_H
 #define SETTINGSOBJECTWRAPPER_H
 
@@ -132,7 +133,6 @@ class TechnicalDetailsSettings : public QObject {
 	Q_PROPERTY(bool show_ccr_sensors  READ showCCRSensors  WRITE setShowCCRSensors  NOTIFY showCCRSensorsChanged)
 	Q_PROPERTY(bool zoomed_plot      READ zoomedPlot      WRITE setZoomedPlot      NOTIFY zoomedPlotChanged)
 	Q_PROPERTY(bool show_sac             READ showSac            WRITE setShowSac            NOTIFY showSacChanged)
-	Q_PROPERTY(bool gf_low_at_maxdepth    READ gfLowAtMaxDepth    WRITE setGfLowAtMaxDepth    NOTIFY gfLowAtMaxDepthChanged)
 	Q_PROPERTY(bool display_unused_tanks READ displayUnusedTanks WRITE setDisplayUnusedTanks NOTIFY displayUnusedTanksChanged)
 	Q_PROPERTY(bool show_average_depth   READ showAverageDepth   WRITE setShowAverageDepth   NOTIFY showAverageDepthChanged)
 	Q_PROPERTY(bool show_pictures_in_profile READ showPicturesInProfile WRITE setShowPicturesInProfile NOTIFY showPicturesInProfileChanged)
@@ -162,7 +162,6 @@ public:
 	bool showCCRSensors() const;
 	bool zoomedPlot() const;
 	bool showSac() const;
-	bool gfLowAtMaxDepth() const;
 	bool displayUnusedTanks() const;
 	bool showAverageDepth() const;
 	bool showPicturesInProfile() const;
@@ -190,7 +189,6 @@ public slots:
 	void setShowCCRSensors(bool value);
 	void setZoomedPlot(bool value);
 	void setShowSac(bool value);
-	void setGfLowAtMaxDepth(bool value);
 	void setDisplayUnusedTanks(bool value);
 	void setShowAverageDepth(bool value);
 	void setShowPicturesInProfile(bool value);
@@ -397,6 +395,7 @@ class DivePlannerSettings : public QObject {
 	Q_PROPERTY(bool display_runtime     READ displayRuntime       WRITE setDisplayRuntime       NOTIFY displayRuntimeChanged)
 	Q_PROPERTY(bool display_duration    READ displayDuration      WRITE setDisplayDuration      NOTIFY displayDurationChanged)
 	Q_PROPERTY(bool display_transitions READ displayTransitions   WRITE setDisplayTransitions   NOTIFY displayTransitionsChanged)
+	Q_PROPERTY(bool display_variations  READ displayVariations    WRITE setDisplayVariations    NOTIFY displayVariationsChanged)
 	Q_PROPERTY(bool doo2breaks          READ doo2breaks           WRITE setDoo2breaks           NOTIFY doo2breaksChanged)
 	Q_PROPERTY(bool drop_stone_mode     READ dropStoneMode        WRITE setDropStoneMode        NOTIFY dropStoneModeChanged)
 	Q_PROPERTY(bool safetystop          READ safetyStop           WRITE setSafetyStop           NOTIFY safetyStopChanged)
@@ -424,6 +423,7 @@ public:
 	bool displayRuntime() const;
 	bool displayDuration() const;
 	bool displayTransitions() const;
+	bool displayVariations() const;
 	bool doo2breaks() const;
 	bool dropStoneMode() const;
 	bool safetyStop() const;
@@ -450,6 +450,7 @@ public slots:
 	void setDisplayRuntime(bool value);
 	void setDisplayDuration(bool value);
 	void setDisplayTransitions(bool value);
+	void setDisplayVariations(bool value);
 	void setDoo2breaks(bool value);
 	void setDropStoneMode(bool value);
 	void setSafetyStop(bool value);
@@ -476,6 +477,7 @@ signals:
 	void displayRuntimeChanged(bool value);
 	void displayDurationChanged(bool value);
 	void displayTransitionsChanged(bool value);
+	void displayVariationsChanged(bool value);
 	void doo2breaksChanged(bool value);
 	void dropStoneModeChanged(bool value);
 	void safetyStopChanged(bool value);
@@ -510,6 +512,8 @@ class UnitsSettings : public QObject {
 	Q_PROPERTY(QString unit_system            READ unitSystem             WRITE setUnitSystem             NOTIFY unitSystemChanged)
 	Q_PROPERTY(bool coordinates_traditional   READ coordinatesTraditional WRITE setCoordinatesTraditional NOTIFY coordinatesTraditionalChanged)
 	Q_PROPERTY(int vertical_speed_time READ verticalSpeedTime    WRITE setVerticalSpeedTime    NOTIFY verticalSpeedTimeChanged)
+	Q_PROPERTY(int duration_units          READ durationUnits        WRITE setDurationUnits         NOTIFY durationUnitChanged)
+	Q_PROPERTY(int show_units_table          READ showUnitsTable        WRITE setShowUnitsTable         NOTIFY showUnitsTableChanged)
 
 public:
 	UnitsSettings(QObject *parent = 0);
@@ -519,6 +523,8 @@ public:
 	int temperature() const;
 	int weight() const;
 	int verticalSpeedTime() const;
+	int durationUnits() const;
+	int showUnitsTable() const;
 	QString unitSystem() const;
 	bool coordinatesTraditional() const;
 
@@ -529,6 +535,8 @@ public slots:
 	void setTemperature(int value);
 	void setWeight(int value);
 	void setVerticalSpeedTime(int value);
+	void setDurationUnits(int value);
+	void setShowUnitsTable(int value);
 	void setUnitSystem(const QString& value);
 	void setCoordinatesTraditional(bool value);
 
@@ -541,6 +549,8 @@ signals:
 	void verticalSpeedTimeChanged(int value);
 	void unitSystemChanged(const QString& value);
 	void coordinatesTraditionalChanged(bool value);
+	void durationUnitChanged(int value);
+	void showUnitsTableChanged(int value);
 private:
 	const QString group = QStringLiteral("Units");
 };
