@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #include "subsurfacestartup.h"
 #include "version.h"
 #include <stdbool.h>
@@ -47,9 +48,9 @@ struct preferences default_prefs = {
 	.display_unused_tanks = false,
 	.show_average_depth = true,
 	.ascrate75 = 9000 / 60,
-	.ascrate50 = 6000 / 60,
-	.ascratestops = 6000 / 60,
-	.ascratelast6m = 1000 / 60,
+	.ascrate50 = 9000 / 60,
+	.ascratestops = 9000 / 60,
+	.ascratelast6m = 9000 / 60,
 	.descrate = 18000 / 60,
 	.sacfactor = 400,
 	.problemsolvingtime = 4,
@@ -65,6 +66,7 @@ struct preferences default_prefs = {
 	.display_runtime = true,
 	.display_duration = true,
 	.display_transitions = true,
+	.display_variations = false,
 	.safetystop = true,
 	.bottomsac = 20000,
 	.decosac = 17000,
@@ -93,7 +95,11 @@ struct preferences default_prefs = {
 	.vpmb_conservatism = 3,
 	.distance_threshold = 1000,
 	.time_threshold = 600,
-	.cloud_timeout = 5
+#if defined(SUBSURFACE_MOBILE)
+	.cloud_timeout = 10,
+#else
+	.cloud_timeout = 5,
+#endif
 };
 
 int run_survey;
