@@ -1,7 +1,11 @@
+// SPDX-License-Identifier: GPL-2.0
 #include <QShortcut>
 #include <QDebug>
 #include <QMessageBox>
 #include <QMenu>
+#include "core/btdiscovery.h"
+
+#include <QBluetoothUuid>
 
 #include "ui_btdeviceselectiondialog.h"
 #include "btdeviceselectiondialog.h"
@@ -148,7 +152,9 @@ void BtDeviceSelectionDialog::on_save_clicked()
 
 	// Save the selected device
 	selectedRemoteDeviceInfo = QSharedPointer<QBluetoothDeviceInfo>(new QBluetoothDeviceInfo(remoteDeviceInfo));
-
+	QString address = remoteDeviceInfo.address().isNull() ? remoteDeviceInfo.deviceUuid().toString() :
+								remoteDeviceInfo.address().toString();
+	saveBtDeviceInfo(address.toUtf8().constData(), remoteDeviceInfo);
 	if (remoteDeviceDiscoveryAgent->isActive()) {
 		// Stop the SDP agent if the clear button is pressed and enable the Scan button
 		remoteDeviceDiscoveryAgent->stop();
@@ -221,7 +227,7 @@ void BtDeviceSelectionDialog::addRemoteDevice(const QBluetoothDeviceInfo &remote
 	QColor pairingColor = QColor(Qt::white);
 #else
 	// By default we use the status label and the color for the UNPAIRED state
-	QColor pairingColor = QColor(Qt::red);
+	QColor pairingColor = QColor("#F1A9A0");
 	QString pairingStatusLabel = tr("UNPAIRED");
 	QBluetoothLocalDevice::Pairing pairingStatus = localDevice->pairingStatus(remoteDeviceInfo.address());
 
@@ -230,10 +236,21 @@ void BtDeviceSelectionDialog::addRemoteDevice(const QBluetoothDeviceInfo &remote
 		pairingColor = QColor(Qt::gray);
 	} else if (pairingStatus == QBluetoothLocalDevice::AuthorizedPaired) {
 		pairingStatusLabel = tr("AUTHORIZED_PAIRED");
-		pairingColor = QColor(Qt::blue);
+		pairingColor = QColor("#89C4F4");
 	}
+	if (remoteDeviceInfo.address().isNull())
+		pairingColor = QColor(Qt::gray);
 
-	QString deviceLabel = tr("%1 (%2)   [State: %3]").arg(remoteDeviceInfo.name(),
+	QString deviceLabel;
+
+#if defined(Q_OS_MACOS) || defined(Q_OS_IOS)
+	if (!remoteDeviceInfo.deviceUuid().isNull()) {
+		// we have only a Uuid, no address, so show that and reset the pairing color
+		deviceLabel = QString("%1 (%2)").arg(remoteDeviceInfo.name(),remoteDeviceInfo.deviceUuid().toString());
+		pairingColor = QColor(Qt::white);
+	} else
+#endif
+	deviceLabel = tr("%1 (%2)   [State: %3]").arg(remoteDeviceInfo.name(),
 							      remoteDeviceInfo.address().toString(),
 							      pairingStatusLabel);
 #endif
@@ -251,16 +268,25 @@ void BtDeviceSelectionDialog::itemClicked(QListWidgetItem *item)
 	// By default we assume that the devices are paired
 	QBluetoothDeviceInfo remoteDeviceInfo = item->data(Qt::UserRole).value<QBluetoothDeviceInfo>();
 	QString statusMessage = tr("The device %1 can be used for connection. You can press the Save button.")
-				  .arg(remoteDeviceInfo.address().toString());
+				  .arg(remoteDeviceInfo.address().isNull() ?
+					       remoteDeviceInfo.deviceUuid().toString() :
+					       remoteDeviceInfo.address().toString());
 	bool enableSaveButton = true;
 
 #if !defined(Q_OS_WIN)
 	// On other platforms than Windows we can obtain the pairing status so if the devices are not paired we disable the button
-	QBluetoothLocalDevice::Pairing pairingStatus = localDevice->pairingStatus(remoteDeviceInfo.address());
+	// except on MacOS for those devices that only give us a Uuid and not and address (as we have no pairing status for those, either)
+	if (!remoteDeviceInfo.address().isNull()) {
+		QBluetoothLocalDevice::Pairing pairingStatus = localDevice->pairingStatus(remoteDeviceInfo.address());
 
-	if (pairingStatus == QBluetoothLocalDevice::Unpaired) {
-		statusMessage = tr("The device %1 must be paired in order to be used. Please use the context menu for pairing options.")
-				  .arg(remoteDeviceInfo.address().toString());
+		if (pairingStatus == QBluetoothLocalDevice::Unpaired) {
+			statusMessage = tr("The device %1 must be paired in order to be used. Please use the context menu for pairing options.")
+					.arg(remoteDeviceInfo.address().toString());
+			enableSaveButton = false;
+		}
+	}
+	if (remoteDeviceInfo.address().isNull() && remoteDeviceInfo.deviceUuid().isNull()) {
+		statusMessage = tr("A device needs a non-zero address for a connection.");
 		enableSaveButton = false;
 	}
 #endif
@@ -416,11 +442,12 @@ void BtDeviceSelectionDialog::deviceDiscoveryError(QBluetoothDeviceDiscoveryAgen
 	ui->dialogStatus->setText(tr("Device discovery error: %1.").arg(errorDescription));
 }
 
+extern QString markBLEAddress(const QBluetoothDeviceInfo *device);
+
 QString BtDeviceSelectionDialog::getSelectedDeviceAddress()
 {
-	if (selectedRemoteDeviceInfo) {
-		return selectedRemoteDeviceInfo.data()->address().toString();
-	}
+	if (selectedRemoteDeviceInfo)
+		return markBLEAddress(selectedRemoteDeviceInfo.data());
 
 	return QString();
 }
