@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 /*
  * divelistview.cpp
  *
@@ -24,9 +25,6 @@
 #include "core/metrics.h"
 #include "core/helpers.h"
 
-//                                #  Date  Rtg Dpth  Dur  Tmp Wght Suit  Cyl  Gas  SAC  OTU  CNS  Px  Loc
-static int defaultWidth[] =    {  70, 140, 90,  50,  50,  50,  50,  70,  50,  50,  70,  50,  50,  5, 500};
-
 DiveListView::DiveListView(QWidget *parent) : QTreeView(parent), mouseClickSelection(false), sortColumn(0),
 	currentOrder(Qt::DescendingOrder), dontEmitDiveChangedSignal(false), selectionSaved(false)
 {
@@ -57,7 +55,7 @@ DiveListView::DiveListView(QWidget *parent) : QTreeView(parent), mouseClickSelec
 
 	// TODO FIXME we need this to get the header names
 	// can we find a smarter way?
-	DiveTripModel *tripModel = new DiveTripModel(this);
+	tripModel = new DiveTripModel(this);
 
 	// set the default width as a minimum between the hard-coded defaults,
 	// the header text width and the (assumed) content width, calculated
@@ -93,8 +91,8 @@ DiveListView::DiveListView(QWidget *parent) : QTreeView(parent), mouseClickSelec
 		if (sw > width)
 			width = sw;
 		width += zw; // small padding
-		if (width > defaultWidth[col])
-			defaultWidth[col] = width;
+		if (width > tripModel->columnWidth(col))
+			tripModel->setColumnWidth(col, width);
 	}
 	delete tripModel;
 
@@ -113,7 +111,7 @@ DiveListView::~DiveListView()
 		if (isColumnHidden(i))
 			continue;
 		// we used to hardcode them all to 100 - so that might still be in the settings
-		if (columnWidth(i) == 100 || columnWidth(i) == defaultWidth[i])
+		if (columnWidth(i) == 100 || columnWidth(i) == tripModel->columnWidth(i))
 			settings.remove(QString("colwidth%1").arg(i));
 		else
 			settings.setValue(QString("colwidth%1").arg(i), columnWidth(i));
@@ -139,7 +137,7 @@ void DiveListView::setupUi()
 		if (width.isValid())
 			setColumnWidth(i, width.toInt());
 		else
-			setColumnWidth(i, defaultWidth[i]);
+			setColumnWidth(i, tripModel->columnWidth(i));
 	}
 	settings.endGroup();
 	if (firstRun)
@@ -429,7 +427,7 @@ void DiveListView::reload(DiveTripModel::Layout layout, bool forceSort)
 	if (oldModel) {
 		oldModel->deleteLater();
 	}
-	DiveTripModel *tripModel = new DiveTripModel(this);
+	tripModel = new DiveTripModel(this);
 	tripModel->setLayout(layout);
 
 	m->setSourceModel(tripModel);
@@ -583,8 +581,8 @@ static bool can_merge(const struct dive *a, const struct dive *b, enum asked_use
 	if (dive_endtime(a) + 30 * 60 < b->when) {
 		if (*have_asked == NOTYET) {
 			if (QMessageBox::warning(MainWindow::instance(),
-						 MainWindow::instance()->tr("Warning"),
-						 MainWindow::instance()->tr("Trying to merge dives with %1min interval in between").arg(
+						 MainWindow::tr("Warning"),
+						 MainWindow::tr("Trying to merge dives with %1min interval in between").arg(
 							 (b->when - dive_endtime(a)) / 60),
 					     QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Cancel) {
 				*have_asked = DONTMERGE;
@@ -916,7 +914,12 @@ void DiveListView::shiftTimes()
 
 void DiveListView::loadImages()
 {
-	QStringList fileNames = QFileDialog::getOpenFileNames(this, tr("Open image files"), lastUsedImageDir(), tr("Image files (*.jpg *.jpeg *.pnm *.tif *.tiff)"));
+	QStringList filters = imageExtensionFilters();
+	QStringList fileNames = QFileDialog::getOpenFileNames(this,
+							      tr("Open image files"),
+							      lastUsedImageDir(),
+							      tr("Image files (%1)").arg(filters.join(" ")));
+
 	if (fileNames.isEmpty())
 		return;
 	updateLastUsedImageDir(QFileInfo(fileNames[0]).dir().path());
