@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #include "desktop-widgets/downloadfromdivecomputer.h"
 #include "core/helpers.h"
 #include "desktop-widgets/mainwindow.h"
@@ -6,42 +7,21 @@
 #include "core/uemis.h"
 #include "core/subsurface-qt/SettingsObjectWrapper.h"
 #include "qt-models/models.h"
+#include "qt-models/diveimportedmodel.h"
 
 #include <QTimer>
 #include <QFileDialog>
 #include <QMessageBox>
 #include <QShortcut>
 
-struct product {
-	const char *product;
-	dc_descriptor_t *descriptor;
-	struct product *next;
-};
-
-struct vendor {
-	const char *vendor;
-	struct product *productlist;
-	struct vendor *next;
-};
-
-struct mydescriptor {
-	const char *vendor;
-	const char *product;
-	dc_family_t type;
-	unsigned int model;
-};
-
 namespace DownloadFromDcGlobal {
 	const char *err_string;
 };
 
-struct dive_table downloadTable;
-
 // Workaround abuse of old libdc types
 #define DC_TRANSPORT_BLUETOOTH 1024
 
 DownloadFromDCWidget::DownloadFromDCWidget(QWidget *parent, Qt::WindowFlags f) : QDialog(parent, f),
-	thread(0),
 	downloading(false),
 	previousLast(0),
 	vendorModel(0),
@@ -51,37 +31,52 @@ DownloadFromDCWidget::DownloadFromDCWidget(QWidget *parent, Qt::WindowFlags f) :
 	ostcFirmwareCheck(0),
 	currentState(INITIAL)
 {
+	diveImportedModel = new DiveImportedModel(this);
+	diveImportedModel->setDiveTable(&downloadTable);
+	vendorModel = new QStringListModel(vendorList);
+	QShortcut *close = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_W), this);
+	QShortcut *quit = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_Q), this);
+
+	int startingWidth = defaultModelFont().pointSize();
+
 	clear_table(&downloadTable);
 	ui.setupUi(this);
 	ui.progressBar->hide();
 	ui.progressBar->setMinimum(0);
 	ui.progressBar->setMaximum(100);
-	diveImportedModel = new DiveImportedModel(this);
 	ui.downloadedView->setModel(diveImportedModel);
 	ui.downloadedView->setSelectionBehavior(QAbstractItemView::SelectRows);
 	ui.downloadedView->setSelectionMode(QAbstractItemView::SingleSelection);
-	int startingWidth = defaultModelFont().pointSize();
 	ui.downloadedView->setColumnWidth(0, startingWidth * 20);
 	ui.downloadedView->setColumnWidth(1, startingWidth * 10);
 	ui.downloadedView->setColumnWidth(2, startingWidth * 10);
-	connect(ui.downloadedView, SIGNAL(clicked(QModelIndex)), diveImportedModel, SLOT(changeSelected(QModelIndex)));
+	ui.chooseDumpFile->setEnabled(ui.dumpToFile->isChecked());
+	ui.chooseLogFile->setEnabled(ui.logToFile->isChecked());
+	ui.selectAllButton->setEnabled(false);
+	ui.unselectAllButton->setEnabled(false);
+	ui.vendor->setModel(vendorModel);
 
 	progress_bar_text = "";
 
-	fill_computer_list();
+	timer->setInterval(200);
 
-	ui.chooseDumpFile->setEnabled(ui.dumpToFile->isChecked());
+	connect(ui.downloadedView, SIGNAL(clicked(QModelIndex)), diveImportedModel, SLOT(changeSelected(QModelIndex)));
 	connect(ui.chooseDumpFile, SIGNAL(clicked()), this, SLOT(pickDumpFile()));
 	connect(ui.dumpToFile, SIGNAL(stateChanged(int)), this, SLOT(checkDumpFile(int)));
-	ui.chooseLogFile->setEnabled(ui.logToFile->isChecked());
 	connect(ui.chooseLogFile, SIGNAL(clicked()), this, SLOT(pickLogFile()));
 	connect(ui.logToFile, SIGNAL(stateChanged(int)), this, SLOT(checkLogFile(int)));
-	ui.selectAllButton->setEnabled(false);
-	ui.unselectAllButton->setEnabled(false);
 	connect(ui.selectAllButton, SIGNAL(clicked()), diveImportedModel, SLOT(selectAll()));
 	connect(ui.unselectAllButton, SIGNAL(clicked()), diveImportedModel, SLOT(selectNone()));
-	vendorModel = new QStringListModel(vendorList);
-	ui.vendor->setModel(vendorModel);
+	connect(timer, SIGNAL(timeout()), this, SLOT(updateProgressBar()));
+	connect(close, SIGNAL(activated()), this, SLOT(close()));
+	connect(quit, SIGNAL(activated()), parent, SLOT(close()));
+
+	connect(&thread, SIGNAL(finished()),
+		this, SLOT(onDownloadThreadFinished()), Qt::QueuedConnection);
+
+	//TODO: Don't call mainwindow.
+	MainWindow *w = MainWindow::instance();
+	connect(&thread, SIGNAL(finished()), w, SLOT(refreshDisplay()));
 
 	auto dc = SettingsObjectWrapper::instance()->dive_computer_settings;
 	if (!dc->dc_vendor().isEmpty()) {
@@ -94,25 +89,18 @@ DownloadFromDCWidget::DownloadFromDCWidget(QWidget *parent, Qt::WindowFlags f) :
 	if (!dc->dc_device().isEmpty())
 		ui.device->setEditText(dc->dc_device());
 
-	timer->setInterval(200);
-	connect(timer, SIGNAL(timeout()), this, SLOT(updateProgressBar()));
 	updateState(INITIAL);
-	memset(&data, 0, sizeof(data));
-	QShortcut *close = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_W), this);
-	connect(close, SIGNAL(activated()), this, SLOT(close()));
-	QShortcut *quit = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_Q), this);
-	connect(quit, SIGNAL(activated()), parent, SLOT(close()));
 	ui.ok->setEnabled(false);
 	ui.downloadCancelRetryButton->setEnabled(true);
 	ui.downloadCancelRetryButton->setText(tr("Download"));
 
-#if defined(BT_SUPPORT) && defined(SSRF_CUSTOM_SERIAL)
+#if defined(BT_SUPPORT) && defined(SSRF_CUSTOM_IO)
 	ui.bluetoothMode->setText(tr("Choose Bluetooth download mode"));
 	ui.bluetoothMode->setChecked(dc->downloadMode() == DC_TRANSPORT_BLUETOOTH);
 	btDeviceSelectionDialog = 0;
-	ui.chooseBluetoothDevice->setEnabled(ui.bluetoothMode->isChecked());
 	connect(ui.bluetoothMode, SIGNAL(stateChanged(int)), this, SLOT(enableBluetoothMode(int)));
 	connect(ui.chooseBluetoothDevice, SIGNAL(clicked()), this, SLOT(selectRemoteBluetoothDevice()));
+	ui.chooseBluetoothDevice->setEnabled(ui.bluetoothMode->isChecked());
 #else
 	ui.bluetoothMode->hide();
 	ui.chooseBluetoothDevice->hide();
@@ -127,12 +115,28 @@ void DownloadFromDCWidget::updateProgressBar()
 		// if we get the first actual text after the download is finished
 		// (which happens for example on the OSTC), then don't bother
 		if (!same_string(progress_bar_text, "") && IS_FP_SAME(progress_bar_fraction, 1.0))
-			progress_bar_text = NULL;
+			progress_bar_text = "";
 	}
 	if (!same_string(progress_bar_text , "")) {
 		ui.progressBar->setFormat(progress_bar_text);
+#if defined(Q_OS_MAC)
+		// on mac the progress bar doesn't show its text
+		ui.progressText->setText(progress_bar_text);
+#endif
 	} else {
-		ui.progressBar->setFormat("%p%");
+		if (IS_FP_SAME(progress_bar_fraction, 0.0)) {
+			ui.progressBar->setFormat(tr("Connecting to dive computer"));
+#if defined(Q_OS_MAC)
+		// on mac the progress bar doesn't show its text
+		ui.progressText->setText(tr("Connecting to dive computer"));
+#endif
+		} else {
+			ui.progressBar->setFormat("%p%");
+#if defined(Q_OS_MAC)
+			// on mac the progress bar doesn't show its text
+			ui.progressText->setText(QString("%1%").arg(lrint(progress_bar_fraction * 100)));
+#endif
+		}
 	}
 	ui.progressBar->setValue(lrint(progress_bar_fraction * 100));
 	free(last_text);
@@ -150,6 +154,10 @@ void DownloadFromDCWidget::updateState(states state)
 		markChildrenAsEnabled();
 		timer->stop();
 		progress_bar_text = "";
+#if defined(Q_OS_MAC)
+		// on mac we show the text in a label
+		ui.progressText->setText(progress_bar_text);
+#endif
 	}
 
 	// tries to cancel an on going download
@@ -172,6 +180,10 @@ void DownloadFromDCWidget::updateState(states state)
 		ui.progressBar->hide();
 		markChildrenAsEnabled();
 		progress_bar_text = "";
+#if defined(Q_OS_MAC)
+		// on mac we show the text in a label
+		ui.progressText->setText(progress_bar_text);
+#endif
 	}
 
 	// DOWNLOAD is finally done, but we don't know if there was an error as libdivecomputer doesn't pass
@@ -184,10 +196,15 @@ void DownloadFromDCWidget::updateState(states state)
 			markChildrenAsEnabled();
 			progress_bar_text = "";
 		} else {
-			progress_bar_text = "";
+			if (downloadTable.nr != 0)
+				progress_bar_text = "";
 			ui.progressBar->setValue(100);
 			markChildrenAsEnabled();
 		}
+#if defined(Q_OS_MAC)
+		// on mac we show the text in a label
+		ui.progressText->setText(progress_bar_text);
+#endif
 	}
 
 	// DOWNLOAD is started.
@@ -203,10 +220,14 @@ void DownloadFromDCWidget::updateState(states state)
 	// got an error
 	else if (state == ERROR) {
 		timer->stop();
-		QMessageBox::critical(this, TITLE_OR_TEXT(tr("Error"), this->thread->error), QMessageBox::Ok);
+		QMessageBox::critical(this, TITLE_OR_TEXT(tr("Error"), thread.error), QMessageBox::Ok);
 		markChildrenAsEnabled();
 		progress_bar_text = "";
 		ui.progressBar->hide();
+#if defined(Q_OS_MAC)
+		// on mac we show the text in a label
+		ui.progressText->setText(progress_bar_text);
+#endif
 	}
 
 	// properly updating the widget state
@@ -247,53 +268,6 @@ void DownloadFromDCWidget::on_product_currentIndexChanged(const QString &product
 	}
 }
 
-void DownloadFromDCWidget::fill_computer_list()
-{
-	dc_iterator_t *iterator = NULL;
-	dc_descriptor_t *descriptor = NULL;
-	struct mydescriptor *mydescriptor;
-
-	QStringList computer;
-	dc_descriptor_iterator(&iterator);
-	while (dc_iterator_next(iterator, &descriptor) == DC_STATUS_SUCCESS) {
-		const char *vendor = dc_descriptor_get_vendor(descriptor);
-		const char *product = dc_descriptor_get_product(descriptor);
-
-		if (!vendorList.contains(vendor))
-			vendorList.append(vendor);
-
-		if (!productList[vendor].contains(product))
-			productList[vendor].push_back(product);
-
-		descriptorLookup[QString(vendor) + QString(product)] = descriptor;
-	}
-	dc_iterator_free(iterator);
-	Q_FOREACH (QString vendor, vendorList)
-		qSort(productList[vendor]);
-
-	/* and add the Uemis Zurich which we are handling internally
-	   THIS IS A HACK as we magically have a data structure here that
-	   happens to match a data structure that is internal to libdivecomputer;
-	   this WILL BREAK if libdivecomputer changes the dc_descriptor struct...
-	   eventually the UEMIS code needs to move into libdivecomputer, I guess */
-
-	mydescriptor = (struct mydescriptor *)malloc(sizeof(struct mydescriptor));
-	mydescriptor->vendor = "Uemis";
-	mydescriptor->product = "Zurich";
-	mydescriptor->type = DC_FAMILY_NULL;
-	mydescriptor->model = 0;
-
-	if (!vendorList.contains("Uemis"))
-		vendorList.append("Uemis");
-
-	if (!productList["Uemis"].contains("Zurich"))
-		productList["Uemis"].push_back("Zurich");
-
-	descriptorLookup["UemisZurich"] = (dc_descriptor_t *)mydescriptor;
-
-	qSort(vendorList);
-}
-
 void DownloadFromDCWidget::on_search_clicked()
 {
 	if (ui.vendor->currentText() == "Uemis") {
@@ -325,60 +299,47 @@ void DownloadFromDCWidget::on_downloadCancelRetryButton_clicked()
 	ui.cancel->setEnabled(false);
 	ui.downloadCancelRetryButton->setText(tr("Cancel download"));
 
-	// I don't really think that create/destroy the thread
-	// is really necessary.
-	if (thread) {
-		thread->deleteLater();
-	}
-
-	data.vendor = strdup(ui.vendor->currentText().toUtf8().data());
-	data.product = strdup(ui.product->currentText().toUtf8().data());
+	auto data = thread.data();
+	data->setVendor(ui.vendor->currentText());
+	data->setProduct(ui.product->currentText());
 #if defined(BT_SUPPORT)
-	data.bluetooth_mode = ui.bluetoothMode->isChecked();
-	if (data.bluetooth_mode && btDeviceSelectionDialog != NULL) {
+	data->setBluetoothMode(ui.bluetoothMode->isChecked());
+	if (data->bluetoothMode() && btDeviceSelectionDialog != NULL) {
 		// Get the selected device address
-		data.devname = strdup(btDeviceSelectionDialog->getSelectedDeviceAddress().toUtf8().data());
+		data->setDevName(btDeviceSelectionDialog->getSelectedDeviceAddress());
 	} else
 		// this breaks an "else if" across lines... not happy...
 #endif
-	if (same_string(data.vendor, "Uemis")) {
+	if (data->vendor() == "Uemis") {
 		char *colon;
 		char *devname = strdup(ui.device->currentText().toUtf8().data());
 
 		if ((colon = strstr(devname, ":\\ (UEMISSDA)")) != NULL) {
 			*(colon + 2) = '\0';
-			fprintf(stderr, "shortened devname to \"%s\"", data.devname);
+			fprintf(stderr, "shortened devname to \"%s\"", devname);
 		}
-		data.devname = devname;
+		data->setDevName(devname);
 	} else {
-		data.devname = strdup(ui.device->currentText().toUtf8().data());
+		data->setDevName(ui.device->currentText());
 	}
-	data.descriptor = descriptorLookup[ui.vendor->currentText() + ui.product->currentText()];
-	data.force_download = ui.forceDownload->isChecked();
-	data.create_new_trip = ui.createNewTrip->isChecked();
-	data.trip = NULL;
-	data.deviceid = data.diveid = 0;
+
+	data->setForceDownload(ui.forceDownload->isChecked());
+	data->setCreateNewTrip(ui.createNewTrip->isChecked());
+	data->setSaveLog(ui.logToFile->isChecked());
+	data->setSaveDump(ui.dumpToFile->isChecked());
 
 	auto dc = SettingsObjectWrapper::instance()->dive_computer_settings;
-	dc->setVendor(data.vendor);
-	dc->setProduct(data.product);
-	dc->setDevice(data.devname);
-#if defined(BT_SUPPORT) && defined(SSRF_CUSTOM_SERIAL)
+	dc->setVendor(data->vendor());
+	dc->setProduct(data->product());
+	dc->setDevice(data->devName());
+
+#if defined(BT_SUPPORT) && defined(SSRF_CUSTOM_IO)
 	dc->setDownloadMode(ui.bluetoothMode->isChecked() ? DC_TRANSPORT_BLUETOOTH : DC_TRANSPORT_SERIAL);
 #endif
 
-	thread = new DownloadThread(this, &data);
-	connect(thread, SIGNAL(finished()),
-		this, SLOT(onDownloadThreadFinished()), Qt::QueuedConnection);
-
-	//TODO: Don't call mainwindow.
-	MainWindow *w = MainWindow::instance();
-	connect(thread, SIGNAL(finished()), w, SLOT(refreshDisplay()));
-
 	// before we start, remember where the dive_table ended
 	previousLast = dive_table.nr;
-
-	thread->start();
+	thread.start();
 
 	// FIXME: We should get the _actual_ device info instead of whatever
 	// the user entered in the dropdown.
@@ -389,8 +350,8 @@ void DownloadFromDCWidget::on_downloadCancelRetryButton_clicked()
 	// We shouldn't do this for memory dumps.
 	if ((product == "OSTC 3" || product == "OSTC 3+" ||
 			product == "OSTC Cr" || product == "OSTC Sport" ||
-			product == "OSTC 4") && !data.libdc_dump)
-		ostcFirmwareCheck = new OstcFirmwareCheck(product);
+			product == "OSTC 4") && ! data->saveDump())
+	ostcFirmwareCheck = new OstcFirmwareCheck(product);
 }
 
 bool DownloadFromDCWidget::preferDownloaded()
@@ -401,8 +362,8 @@ bool DownloadFromDCWidget::preferDownloaded()
 void DownloadFromDCWidget::checkLogFile(int state)
 {
 	ui.chooseLogFile->setEnabled(state == Qt::Checked);
-	data.libdc_log = (state == Qt::Checked);
-	if (state == Qt::Checked && logFile.isEmpty()) {
+	// TODO: Verify the Thread.
+	if (state == Qt::Checked) {
 		pickLogFile();
 	}
 }
@@ -412,21 +373,19 @@ void DownloadFromDCWidget::pickLogFile()
 	QString filename = existing_filename ?: prefs.default_filename;
 	QFileInfo fi(filename);
 	filename = fi.absolutePath().append(QDir::separator()).append("subsurface.log");
-	logFile = QFileDialog::getSaveFileName(this, tr("Choose file for dive computer download logfile"),
+	QString logFile = QFileDialog::getSaveFileName(this, tr("Choose file for dive computer download logfile"),
 					       filename, tr("Log files (*.log)"));
 	if (!logFile.isEmpty()) {
 		free(logfile_name);
-		logfile_name = strdup(logFile.toUtf8().data());
+		logfile_name = copy_string(logFile.toUtf8().data());
 	}
 }
 
 void DownloadFromDCWidget::checkDumpFile(int state)
 {
 	ui.chooseDumpFile->setEnabled(state == Qt::Checked);
-	data.libdc_dump = (state == Qt::Checked);
 	if (state == Qt::Checked) {
-		if (dumpFile.isEmpty())
-			pickDumpFile();
+		pickDumpFile();
 		if (!dumpWarningShown) {
 			QMessageBox::warning(this, tr("Warning"),
 					     tr("Saving the libdivecomputer dump will NOT download dives to the dive list."));
@@ -440,11 +399,11 @@ void DownloadFromDCWidget::pickDumpFile()
 	QString filename = existing_filename ?: prefs.default_filename;
 	QFileInfo fi(filename);
 	filename = fi.absolutePath().append(QDir::separator()).append("subsurface.bin");
-	dumpFile = QFileDialog::getSaveFileName(this, tr("Choose file for dive computer binary dump file"),
+	QString dumpFile = QFileDialog::getSaveFileName(this, tr("Choose file for dive computer binary dump file"),
 						filename, tr("Dump files (*.bin)"));
 	if (!dumpFile.isEmpty()) {
 		free(dumpfile_name);
-		dumpfile_name = strdup(dumpFile.toUtf8().data());
+		dumpfile_name = copy_string(dumpFile.toUtf8().data());
 	}
 }
 
@@ -459,7 +418,7 @@ void DownloadFromDCWidget::reject()
 void DownloadFromDCWidget::onDownloadThreadFinished()
 {
 	if (currentState == DOWNLOADING) {
-		if (thread->error.isEmpty())
+		if (thread.error.isEmpty())
 			updateState(DONE);
 		else
 			updateState(ERROR);
@@ -472,7 +431,6 @@ void DownloadFromDCWidget::onDownloadThreadFinished()
 	if (downloadTable.nr) {
 		diveImportedModel->setImportedDivesIndexes(0, downloadTable.nr - 1);
 	}
-
 }
 
 void DownloadFromDCWidget::on_cancel_clicked()
@@ -496,6 +454,8 @@ void DownloadFromDCWidget::on_ok_clicked()
 	for (int i = 0; i < downloadTable.nr; i++) {
 		if (diveImportedModel->data(diveImportedModel->index(i, 0),Qt::CheckStateRole) == Qt::Checked)
 			record_dive(downloadTable.dives[i]);
+		else
+			clear_dive(downloadTable.dives[i]);
 		downloadTable.dives[i] = NULL;
 	}
 	downloadTable.nr = 0;
@@ -519,8 +479,9 @@ void DownloadFromDCWidget::on_ok_clicked()
 		MainWindow::instance()->dive_list()->selectDive(idx, true);
 	}
 
-	if (ostcFirmwareCheck && currentState == DONE)
-		ostcFirmwareCheck->checkLatest(this, &data);
+	if (ostcFirmwareCheck && currentState == DONE) {
+		ostcFirmwareCheck->checkLatest(this, thread.data()->internalData());
+	}
 	accept();
 }
 
@@ -622,143 +583,3 @@ void DownloadFromDCWidget::fill_device_list(int dc_type)
 		ui.device->setCurrentIndex(deviceIndex);
 }
 
-DownloadThread::DownloadThread(QObject *parent, device_data_t *data) : QThread(parent),
-	data(data)
-{
-}
-
-static QString str_error(const char *fmt, ...)
-{
-	va_list args;
-	va_start(args, fmt);
-	const QString str = QString().vsprintf(fmt, args);
-	va_end(args);
-
-	return str;
-}
-
-void DownloadThread::run()
-{
-	const char *errorText;
-	import_thread_cancelled = false;
-	data->download_table = &downloadTable;
-	if (!strcmp(data->vendor, "Uemis"))
-		errorText = do_uemis_import(data);
-	else
-		errorText = do_libdivecomputer_import(data);
-	if (errorText)
-		error = str_error(errorText, data->devname, data->vendor, data->product);
-}
-
-DiveImportedModel::DiveImportedModel(QObject *o) : QAbstractTableModel(o),
-	firstIndex(0),
-	lastIndex(-1),
-	checkStates(0)
-{
-}
-
-int DiveImportedModel::columnCount(const QModelIndex &model) const
-{
-	Q_UNUSED(model)
-	return 3;
-}
-
-int DiveImportedModel::rowCount(const QModelIndex &model) const
-{
-	Q_UNUSED(model)
-	return lastIndex - firstIndex + 1;
-}
-
-QVariant DiveImportedModel::headerData(int section, Qt::Orientation orientation, int role) const
-{
-	if (orientation == Qt::Vertical)
-		return QVariant();
-	if (role == Qt::DisplayRole) {
-		switch (section) {
-		case 0:
-			return QVariant(tr("Date/time"));
-		case 1:
-			return QVariant(tr("Duration"));
-		case 2:
-			return QVariant(tr("Depth"));
-		}
-	}
-	return QVariant();
-}
-
-QVariant DiveImportedModel::data(const QModelIndex &index, int role) const
-{
-	if (!index.isValid())
-		return QVariant();
-
-	if (index.row() + firstIndex > lastIndex)
-		return QVariant();
-
-	struct dive *d = get_dive_from_table(index.row() + firstIndex, &downloadTable);
-	if (!d)
-		return QVariant();
-	if (role == Qt::DisplayRole) {
-		switch (index.column()) {
-		case 0:
-			return QVariant(get_short_dive_date_string(d->when));
-		case 1:
-			return QVariant(get_dive_duration_string(d->duration.seconds, tr("h:"), tr("min")));
-		case 2:
-			return QVariant(get_depth_string(d->maxdepth.mm, true, false));
-		}
-	}
-	if (role == Qt::CheckStateRole) {
-		if (index.column() == 0)
-			return checkStates[index.row()] ? Qt::Checked : Qt::Unchecked;
-	}
-	return QVariant();
-}
-
-void DiveImportedModel::changeSelected(QModelIndex clickedIndex)
-{
-	checkStates[clickedIndex.row()] = !checkStates[clickedIndex.row()];
-	dataChanged(index(clickedIndex.row(), 0), index(clickedIndex.row(), 0), QVector<int>() << Qt::CheckStateRole);
-}
-
-void DiveImportedModel::selectAll()
-{
-	memset(checkStates, true, lastIndex - firstIndex + 1);
-	dataChanged(index(0, 0), index(lastIndex - firstIndex, 0), QVector<int>() << Qt::CheckStateRole);
-}
-
-void DiveImportedModel::selectNone()
-{
-	memset(checkStates, false, lastIndex - firstIndex + 1);
-	dataChanged(index(0, 0), index(lastIndex - firstIndex,0 ), QVector<int>() << Qt::CheckStateRole);
-}
-
-Qt::ItemFlags DiveImportedModel::flags(const QModelIndex &index) const
-{
-	if (index.column() != 0)
-		return QAbstractTableModel::flags(index);
-	return QAbstractTableModel::flags(index) | Qt::ItemIsUserCheckable;
-}
-
-void DiveImportedModel::clearTable()
-{
-	beginRemoveRows(QModelIndex(), 0, lastIndex - firstIndex);
-	lastIndex = -1;
-	firstIndex = 0;
-	endRemoveRows();
-}
-
-void DiveImportedModel::setImportedDivesIndexes(int first, int last)
-{
-	Q_ASSERT(last >= first);
-	if (lastIndex >= firstIndex) {
-		beginRemoveRows(QModelIndex(), 0, lastIndex - firstIndex);
-		endRemoveRows();
-	}
-	beginInsertRows(QModelIndex(), 0, last - first);
-	lastIndex = last;
-	firstIndex = first;
-	delete[] checkStates;
-	checkStates = new bool[last - first + 1];
-	memset(checkStates, true, last - first + 1);
-	endInsertRows();
-}
