@@ -1,9 +1,10 @@
+// SPDX-License-Identifier: GPL-2.0
 #include "desktop-widgets/locationinformation.h"
 #include "core/dive.h"
 #include "desktop-widgets/mainwindow.h"
 #include "desktop-widgets/divelistview.h"
 #include "core/qthelper.h"
-#include "desktop-widgets/globe.h"
+#include "desktop-widgets/mapwidget.h"
 #include "qt-models/filtermodels.h"
 #include "qt-models/divelocationmodel.h"
 #include "core/divesitehelpers.h"
@@ -42,17 +43,15 @@ LocationInformationWidget::LocationInformationWidget(QWidget *parent) : QGroupBo
 	ui.diveSiteListView->setModel(filter_model);
 	ui.diveSiteListView->setModelColumn(LocationInformationModel::NAME);
 	ui.diveSiteListView->installEventFilter(this);
-#ifndef NO_MARBLE
-	// Globe Management Code.
+	// Map Management Code.
 	connect(this, &LocationInformationWidget::requestCoordinates,
-		GlobeGPS::instance(), &GlobeGPS::prepareForGetDiveCoordinates);
+		MapWidget::instance(), &MapWidget::prepareForGetDiveCoordinates);
 	connect(this, &LocationInformationWidget::endRequestCoordinates,
-		GlobeGPS::instance(), &GlobeGPS::endGetDiveCoordinates);
-	connect(GlobeGPS::instance(), &GlobeGPS::coordinatesChanged,
+		MapWidget::instance(), &MapWidget::endGetDiveCoordinates);
+	connect(MapWidget::instance(), &MapWidget::coordinatesChanged,
 		this, &LocationInformationWidget::updateGpsCoordinates);
 	connect(this, &LocationInformationWidget::endEditDiveSite,
-		GlobeGPS::instance(), &GlobeGPS::repopulateLabels);
-#endif
+		MapWidget::instance(), &MapWidget::repopulateLabels);
 }
 
 bool LocationInformationWidget::eventFilter(QObject *, QEvent *ev)
@@ -94,6 +93,11 @@ void LocationInformationWidget::updateLabels()
 		ui.diveSiteName->setText(displayed_dive_site.name);
 	else
 		ui.diveSiteName->clear();
+	const char *country = taxonomy_get_country(&displayed_dive_site.taxonomy);
+	if (country)
+		ui.diveSiteCountry->setText(country);
+	else
+		ui.diveSiteCountry->clear();
 	if (displayed_dive_site.description)
 		ui.diveSiteDescription->setText(displayed_dive_site.description);
 	else
@@ -110,10 +114,18 @@ void LocationInformationWidget::updateLabels()
 		ui.diveSiteCoordinates->clear();
 	}
 
-	ui.locationTags->setText(constructLocationTags(displayed_dive_site.uuid));
+	ui.locationTags->setText(constructLocationTags(&displayed_dive_site, false));
 
-	emit startFilterDiveSite(displayed_dive_site.uuid);
-	emit startEditDiveSite(displayed_dive_site.uuid);
+}
+
+void LocationInformationWidget::clearLabels()
+{
+	ui.diveSiteName->clear();
+	ui.diveSiteCountry->clear();
+	ui.diveSiteDescription->clear();
+	ui.diveSiteNotes->clear();
+	ui.diveSiteCoordinates->clear();
+	ui.locationTags->clear();
 }
 
 void LocationInformationWidget::updateGpsCoordinates()
@@ -121,6 +133,7 @@ void LocationInformationWidget::updateGpsCoordinates()
 	QString oldText = ui.diveSiteCoordinates->text();
 	const char *coords = printGPSCoords(displayed_dive_site.latitude.udeg, displayed_dive_site.longitude.udeg);
 	ui.diveSiteCoordinates->setText(coords);
+	ui.geoCodeButton->setEnabled(dive_site_has_gps_location(&displayed_dive_site));
 	free((void *)coords);
 	if (oldText != ui.diveSiteCoordinates->text())
 		markChangedWidget(ui.diveSiteCoordinates);
@@ -130,34 +143,53 @@ void LocationInformationWidget::acceptChanges()
 {
 	char *uiString;
 	struct dive_site *currentDs;
-	uiString = ui.diveSiteName->text().toUtf8().data();
-
-	if (get_dive_site_by_uuid(displayed_dive_site.uuid) != NULL)
+	uiString = copy_string(ui.diveSiteName->text().toUtf8().data());
+	if (get_dive_site_by_uuid(displayed_dive_site.uuid) != NULL) {
 		currentDs = get_dive_site_by_uuid(displayed_dive_site.uuid);
-	else
+	} else {
+		qWarning() << "did not have valid dive site in LocationInformationWidget";
 		currentDs = get_dive_site_by_uuid(create_dive_site_from_current_dive(uiString));
-
+		displayed_dive.dive_site_uuid = currentDs->uuid;
+	}
 	currentDs->latitude = displayed_dive_site.latitude;
 	currentDs->longitude = displayed_dive_site.longitude;
 	if (!same_string(uiString, currentDs->name)) {
 		free(currentDs->name);
-		currentDs->name = copy_string(uiString);
+		currentDs->name = uiString;
+	} else {
+		free(uiString);
 	}
-	uiString = ui.diveSiteDescription->text().toUtf8().data();
+	uiString = copy_string(ui.diveSiteDescription->text().toUtf8().data());
 	if (!same_string(uiString, currentDs->description)) {
 		free(currentDs->description);
-		currentDs->description = copy_string(uiString);
+		currentDs->description = uiString;
+	} else {
+		free(uiString);
 	}
-	uiString = ui.diveSiteNotes->document()->toPlainText().toUtf8().data();
+	uiString = copy_string(ui.diveSiteCountry->text().toUtf8().data());
+	// if the user entered a different contriy, first update the taxonomy
+	// for the displayed dive site; this below will get copied into the currentDs
+	if (!same_string(uiString, taxonomy_get_country(&displayed_dive_site.taxonomy)) &&
+	    !same_string(uiString, ""))
+		taxonomy_set_country(&displayed_dive_site.taxonomy, uiString, taxonomy_origin::GEOMANUAL);
+	else
+		free(uiString);
+	// now update the currentDs (which we then later copy back ontop of displayed_dive_site
+	copy_dive_site_taxonomy(&displayed_dive_site, currentDs);
+
+	uiString = copy_string(ui.diveSiteNotes->document()->toPlainText().toUtf8().data());
 	if (!same_string(uiString, currentDs->notes)) {
 		free(currentDs->notes);
-		currentDs->notes = copy_string(uiString);
+		currentDs->notes = uiString;
+	} else {
+		free(uiString);
 	}
+
 	if (!ui.diveSiteCoordinates->text().isEmpty()) {
 		double lat, lon;
 		parseGpsText(ui.diveSiteCoordinates->text(), &lat, &lon);
-		currentDs->latitude.udeg = (int)(lat * 1000000.0);
-		currentDs->longitude.udeg = (int)(lon * 1000000.0);
+		currentDs->latitude.udeg = lrint(lat * 1000000.0);
+		currentDs->longitude.udeg = lrint(lon * 1000000.0);
 	}
 	if (dive_site_is_empty(currentDs)) {
 		LocationInformationModel::instance()->removeRow(get_divesite_idx(currentDs));
@@ -190,6 +222,8 @@ void LocationInformationWidget::showEvent(QShowEvent *ev)
 		emit startFilterDiveSite(displayed_dive_site.uuid);
 		if (m)
 			m->invalidate();
+	} else {
+		clearLabels();
 	}
 	emit requestCoordinates();
 
@@ -232,8 +266,8 @@ void LocationInformationWidget::on_diveSiteCoordinates_textChanged(const QString
 	if (!same_string(qPrintable(text), coords)) {
 		double latitude, longitude;
 		if (parseGpsText(text, &latitude, &longitude)) {
-			displayed_dive_site.latitude.udeg = (int)(latitude * 1000000);
-			displayed_dive_site.longitude.udeg = (int)(longitude * 1000000);
+			displayed_dive_site.latitude.udeg = lrint(latitude * 1000000);
+			displayed_dive_site.longitude.udeg = lrint(longitude * 1000000);
 			markChangedWidget(ui.diveSiteCoordinates);
 			emit coordinatesChanged();
 			ui.geoCodeButton->setEnabled(latitude != 0 && longitude != 0);
@@ -244,6 +278,12 @@ void LocationInformationWidget::on_diveSiteCoordinates_textChanged(const QString
 	free((void *)coords);
 }
 
+void LocationInformationWidget::on_diveSiteCountry_textChanged(const QString& text)
+{
+	if (!same_string(qPrintable(text), taxonomy_get_country(&displayed_dive_site.taxonomy)))
+		markChangedWidget(ui.diveSiteCountry);
+}
+
 void LocationInformationWidget::on_diveSiteDescription_textChanged(const QString &text)
 {
 	if (!same_string(qPrintable(text), displayed_dive_site.description))
@@ -267,6 +307,7 @@ void LocationInformationWidget::resetPallete()
 	QPalette p;
 	ui.diveSiteCoordinates->setPalette(p);
 	ui.diveSiteDescription->setPalette(p);
+	ui.diveSiteCountry->setPalette(p);
 	ui.diveSiteName->setPalette(p);
 	ui.diveSiteNotes->setPalette(p);
 }
@@ -274,7 +315,7 @@ void LocationInformationWidget::resetPallete()
 void LocationInformationWidget::reverseGeocode()
 {
 	ReverseGeoLookupThread *geoLookup = ReverseGeoLookupThread::instance();
-	geoLookup->lookup(&displayed_dive_site);
+	geoLookup->run();
 	updateLabels();
 }
 
