@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 /*
  * mainwindow.cpp
  *
@@ -11,6 +12,7 @@
 #include <QSettings>
 #include <QShortcut>
 #include <QToolBar>
+#include <QStatusBar>
 
 #include "core/version.h"
 #include "desktop-widgets/divelistview.h"
@@ -22,7 +24,6 @@
 #include "core/planner.h"
 #include "qt-models/filtermodels.h"
 #include "profile-widget/profilewidget2.h"
-#include "desktop-widgets/globe.h"
 #include "core/divecomputer.h"
 #include "desktop-widgets/tab-widgets/maintab.h"
 #include "desktop-widgets/diveplanner.h"
@@ -62,19 +63,23 @@
 #include "plugins/facebook/facebookconnectwidget.h"
 #endif
 
+#include "desktop-widgets/mapwidget.h"
+
 QProgressDialog *progressDialog = NULL;
 bool progressDialogCanceled = false;
 
-extern "C" int updateProgress(bool reset, const char *text)
-{
-	static int percent;
+static int progressCounter = 0;
 
-	if (reset)
-		percent = 0;
+extern "C" int updateProgress(const char *text)
+{
 	if (verbose)
-		qDebug() << "git storage:" << +percent << "% (" << text << ")";
-	if (progressDialog)
-		progressDialog->setValue(percent);
+		qDebug() << "git storage:" << text;
+	if (progressDialog) {
+		progressDialog->setLabelText(text);
+		progressDialog->setValue(++progressCounter);
+		if (progressCounter == 100)
+			progressCounter = 0; // yes this is silly, but we really don't know how long it will take
+	}
 	qApp->processEvents();
 	return progressDialogCanceled;
 }
@@ -99,12 +104,7 @@ MainWindow::MainWindow() : QMainWindow(),
 	MainTab *mainTab = new MainTab();
 	DiveListView *diveListView = new DiveListView();
 	ProfileWidget2 *profileWidget = new ProfileWidget2();
-
-#ifndef NO_MARBLE
-	GlobeGPS *globeGps = GlobeGPS::instance();
-#else
-	QWidget *globeGps = NULL;
-#endif
+	MapWidget *mapWidget = MapWidget::instance();
 
 	PlannerSettingsWidget *plannerSettings = new PlannerSettingsWidget();
 	DivePlannerWidget *plannerWidget = new DivePlannerWidget();
@@ -154,12 +154,12 @@ MainWindow::MainWindow() : QMainWindow(),
 	enabledList.push_back(enabled);
 	disabledList.push_back(disabled);
 
-	registerApplicationState("Default", mainTab, profileContainer, diveListView, globeGps );
-	registerApplicationState("AddDive", mainTab, profileContainer, diveListView, globeGps );
-	registerApplicationState("EditDive", mainTab, profileContainer, diveListView, globeGps );
+	registerApplicationState("Default", mainTab, profileContainer, diveListView, mapWidget );
+	registerApplicationState("AddDive", mainTab, profileContainer, diveListView, mapWidget );
+	registerApplicationState("EditDive", mainTab, profileContainer, diveListView, mapWidget );
 	registerApplicationState("PlanDive", plannerWidget, profileContainer, plannerSettings, plannerDetails );
-	registerApplicationState("EditPlannedDive", plannerWidget, profileContainer, diveListView, globeGps );
-	registerApplicationState("EditDiveSite", diveSiteEdit, profileContainer, diveListView, globeGps);
+	registerApplicationState("EditPlannedDive", plannerWidget, profileContainer, diveListView, mapWidget );
+	registerApplicationState("EditDiveSite", diveSiteEdit, profileContainer, diveListView, mapWidget);
 
 	setStateProperties("Default", enabledList, enabledList, enabledList,enabledList);
 	setStateProperties("AddDive", enabledList, enabledList, enabledList,enabledList);
@@ -194,10 +194,8 @@ MainWindow::MainWindow() : QMainWindow(),
 	connect(DivePlannerPointsModel::instance(), SIGNAL(planCanceled()), this, SLOT(planCanceled()));
 	connect(plannerDetails->printPlan(), SIGNAL(pressed()), divePlannerWidget(), SLOT(printDecoPlan()));
 	connect(this, SIGNAL(startDiveSiteEdit()), this, SLOT(on_actionDiveSiteEdit_triggered()));
+	connect(information(), SIGNAL(diveSiteChanged(struct dive_site *)), mapWidget, SLOT(centerOnDiveSite(struct dive_site *)));
 
-#ifndef NO_MARBLE
-	connect(information(), SIGNAL(diveSiteChanged(struct dive_site *)), globeGps, SLOT(centerOnDiveSite(struct dive_site *)));
-#endif
 	wtu = new WindowTitleUpdate();
 	connect(WindowTitleUpdate::instance(), SIGNAL(updateTitle()), this, SLOT(setAutomaticTitle()));
 #ifdef NO_PRINTING
@@ -213,14 +211,11 @@ MainWindow::MainWindow() : QMainWindow(),
 	diveListView->reload(DiveTripModel::TREE);
 	diveListView->reloadHeaderActions();
 	diveListView->setFocus();
-	GlobeGPS::instance()->reload();
+	MapWidget::instance()->reload();
 	diveListView->expand(dive_list()->model()->index(0, 0));
 	diveListView->scrollTo(dive_list()->model()->index(0, 0), QAbstractItemView::PositionAtCenter);
 	divePlannerWidget()->settingsChanged();
 	divePlannerSettingsWidget()->settingsChanged();
-#ifdef NO_MARBLE
-	ui.menuView->removeAction(ui.actionViewGlobe);
-#endif
 #ifdef NO_USERMANUAL
 	ui.menuHelp->removeAction(ui.actionUserManual);
 #endif
@@ -318,6 +313,14 @@ MainWindow::MainWindow() : QMainWindow(),
 	ui.profTankbar->setChecked(sWrapper->techDetails->tankBar());
 	ui.profTissues->setChecked(sWrapper->techDetails->percentageGraph());
 	ui.profScaled->setChecked(sWrapper->techDetails->zoomedPlot());
+
+// full screen support is buggy on Windows and Ubuntu.
+// require the FULLSCREEN_SUPPORT macro to enable it!
+#ifndef FULLSCREEN_SUPPORT
+	ui.actionFullScreen->setEnabled(false);
+	ui.actionFullScreen->setVisible(false);
+	setWindowState(windowState() & ~Qt::WindowFullScreen);
+#endif
 }
 
 MainWindow::~MainWindow()
@@ -340,6 +343,9 @@ void MainWindow::setupSocialNetworkMenu()
 	FacebookManager *fb = FacebookManager::instance();
 	connect(fb, &FacebookManager::justLoggedIn, this, &MainWindow::facebookLoggedIn);
 	connect(fb, &FacebookManager::justLoggedOut, this, &MainWindow::facebookLoggedOut);
+	connect(fb, &FacebookManager::sendMessage, [this](const QString& msg) {
+		statusBar()->showMessage(msg, 10000); // show message for 10 secs on the statusbar.
+	});
 	share_on_fb = new QAction(this);
 	share_on_fb->setText(facebookPlugin->socialNetworkName());
 	share_on_fb->setIcon(QIcon(facebookPlugin->socialNetworkIcon()));
@@ -368,6 +374,7 @@ void MainWindow::facebookLoggedOut()
 
 void MainWindow::socialNetworkRequestConnect()
 {
+	qDebug() << "Requesting connect on the social network";
 	QAction *action = qobject_cast<QAction*>(sender());
 	ISocialNetworkIntegration *plugin = qobject_cast<ISocialNetworkIntegration*>(action->data().value<QObject*>());
 	if (plugin->isConnected())
@@ -425,7 +432,7 @@ void MainWindow::refreshDisplay(bool doRecreateDiveList)
 	getNotificationWidget()->showNotification(get_error_string(), KMessageWidget::Error);
 	information()->reload();
 	TankInfoModel::instance()->update();
-	GlobeGPS::instance()->reload();
+	MapWidget::instance()->reload();
 	if (doRecreateDiveList)
 		recreateDiveList();
 
@@ -496,7 +503,7 @@ void MainWindow::current_dive_changed(int divenr)
 	graphics()->plotDive();
 	information()->updateDiveInfo();
 	configureToolbar();
-	GlobeGPS::instance()->reload();
+	MapWidget::instance()->reload();
 }
 
 void MainWindow::on_actionNew_triggered()
@@ -583,6 +590,10 @@ void MainWindow::on_actionCloudstorageopen_triggered()
 void MainWindow::on_actionCloudstoragesave_triggered()
 {
 	QString filename;
+	if (!dive_table.nr) {
+		getNotificationWidget()->showNotification(tr("Don't save an empty log to the cloud"), KMessageWidget::Error);
+		return;
+	}
 	if (getCloudURL(filename)) {
 		getNotificationWidget()->showNotification(get_error_string(), KMessageWidget::Error);
 		return;
@@ -652,7 +663,7 @@ void MainWindow::cleanUpEmpty()
 	information()->updateDiveInfo(true);
 	graphics()->setEmptyState();
 	dive_list()->reload(DiveTripModel::TREE);
-	GlobeGPS::instance()->reload();
+	MapWidget::instance()->reload();
 	if (!existing_filename)
 		setTitle(MWTF_DEFAULT);
 	disableShortcuts();
@@ -778,7 +789,6 @@ void MainWindow::on_actionQuit_triggered()
 void MainWindow::on_actionDownloadDC_triggered()
 {
 	DownloadFromDCWidget dlg(this);
-
 	dlg.exec();
 }
 
@@ -907,6 +917,7 @@ void MainWindow::setupForAddAndPlan(const char *model)
 	displayed_dive.id = dive_getUniqID(&displayed_dive);
 	displayed_dive.when = QDateTime::currentMSecsSinceEpoch() / 1000L + gettimezoneoffset() + 3600;
 	displayed_dive.dc.model = strdup(model); // don't translate! this is stored in the XML file
+	dc_number = 1;
 	// setup the dive cylinders
 	DivePlannerPointsModel::instance()->clear();
 	DivePlannerPointsModel::instance()->setupCylinders();
@@ -1017,7 +1028,7 @@ void MainWindow::on_actionEditDive_triggered()
 	disableShortcuts();
 	DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::ADD);
 	graphics()->setAddState();
-	GlobeGPS::instance()->endGetDiveCoordinates();
+	MapWidget::instance()->endGetDiveCoordinates();
 	setApplicationState("EditDive");
 	DivePlannerPointsModel::instance()->loadFromDive(current_dive);
 	information()->enableEdition(MainTab::MANUALLY_ADDED_DIVE);
@@ -1095,10 +1106,10 @@ void MainWindow::on_actionViewInfo_triggered()
 	ui.mainSplitter->setSizes(BEHAVIOR << EXPANDED << COLLAPSED);
 }
 
-void MainWindow::on_actionViewGlobe_triggered()
+void MainWindow::on_actionViewMap_triggered()
 {
 	TOGGLE_COLLAPSABLE( true );
-	beginChangeState(GLOBE_MAXIMIZED);
+	beginChangeState(MAP_MAXIMIZED);
 	ui.mainSplitter->setSizes(BEHAVIOR << COLLAPSED << EXPANDED);
 	ui.bottomSplitter->setSizes(BEHAVIOR << COLLAPSED << EXPANDED);
 }
@@ -1319,8 +1330,8 @@ void MainWindow::initialUiSetup()
 	case VIEWALL:
 		on_actionViewAll_triggered();
 		break;
-	case GLOBE_MAXIMIZED:
-		on_actionViewGlobe_triggered();
+	case MAP_MAXIMIZED:
+		on_actionViewMap_triggered();
 		break;
 	case INFO_MAXIMIZED:
 		on_actionViewInfo_triggered();
@@ -2015,8 +2026,9 @@ void MainWindow::showProgressBar()
 
 	progressDialog = new QProgressDialog(tr("Contacting cloud service..."), tr("Cancel"), 0, 100, this);
 	progressDialog->setWindowModality(Qt::WindowModal);
-	progressDialog->setMinimumDuration(200);
+	progressDialog->setMinimumDuration(0);
 	progressDialogCanceled = false;
+	progressCounter = 0;
 	connect(progressDialog, SIGNAL(canceled()), this, SLOT(cancelCloudStorageOperation()));
 }
 
