@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 /*
  * mainwindow.h
  *
@@ -18,18 +19,11 @@
 #include "core/windowtitleupdate.h"
 #include "core/gpslocation.h"
 
-struct DiveList;
 class QSortFilterProxyModel;
 class DiveTripModel;
-
-class DiveInfo;
-class DiveNotes;
-class Stats;
-class Equipment;
 class QItemSelection;
 class DiveListView;
 class MainTab;
-class ProfileGraphicsView;
 class QWebView;
 class QSettings;
 class UpdateManager;
@@ -59,7 +53,7 @@ public:
 
 	enum CurrentState {
 		VIEWALL,
-		GLOBE_MAXIMIZED,
+		MAP_MAXIMIZED,
 		INFO_MAXIMIZED,
 		PROFILE_MAXIMIZED,
 		LIST_MAXIMIZED
@@ -127,7 +121,7 @@ slots:
 	void on_actionViewList_triggered();
 	void on_actionViewProfile_triggered();
 	void on_actionViewInfo_triggered();
-	void on_actionViewGlobe_triggered();
+	void on_actionViewMap_triggered();
 	void on_actionViewAll_triggered();
 	void on_actionPreviousDC_triggered();
 	void on_actionNextDC_triggered();
