@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #include "facebook_integration.h"
 #include "facebookconnectwidget.h"
 
@@ -39,5 +40,5 @@ void FacebookPlugin::requestUpload()
 {
 	FacebookManager *instance = FacebookManager::instance();
 	if (instance->loggedIn())
-		instance->sendDive();
+		instance->sendDiveInit();
 }
