@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #include "preferences_graph.h"
 #include "ui_preferences_graph.h"
 #include "core/prefs-macros.h"
@@ -37,7 +38,6 @@ void PreferencesGraph::refreshSettings()
 	ui->gflow->setValue(prefs.gflow);
 	ui->gfhigh->setValue(prefs.gfhigh);
 	ui->vpmb_conservatism->setValue(prefs.vpmb_conservatism);
-	ui->gf_low_at_maxdepth->setChecked(prefs.gf_low_at_maxdepth);
 	ui->show_ccr_setpoint->setChecked(prefs.show_ccr_setpoint);
 	ui->show_ccr_sensors->setChecked(prefs.show_ccr_sensors);
 	ui->defaultSetpoint->setValue((double)prefs.defaultsetpoint / 1000.0);
@@ -68,7 +68,6 @@ void PreferencesGraph::syncSettings()
 	tech->setGflow(ui->gflow->value());
 	tech->setGfhigh(ui->gfhigh->value());
 	tech->setVpmbConservatism(ui->vpmb_conservatism->value());
-	tech->setGfLowAtMaxDepth(ui->gf_low_at_maxdepth->isChecked());
 	tech->setShowCCRSetpoint(ui->show_ccr_setpoint->isChecked());
 	tech->setShowCCRSensors(ui->show_ccr_sensors->isChecked());
 	tech->setDisplayUnusedTanks(ui->display_unused_tanks->isChecked());
@@ -92,7 +91,6 @@ void PreferencesGraph::on_buehlmann_toggled(bool buehlmann)
 	ui->gflow->setEnabled(buehlmann);
 	ui->label_GFhigh->setEnabled(buehlmann);
 	ui->label_GFlow->setEnabled(buehlmann);
-	ui->gf_low_at_maxdepth->setEnabled(buehlmann);
 	ui->vpmb_conservatism->setEnabled(!buehlmann);
 	ui->label_VPMB->setEnabled(!buehlmann);
 }
