@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #include "desktop-widgets/simplewidgets.h"
 #include "qt-models/filtermodels.h"
 
@@ -18,6 +19,7 @@
 #include "core/display.h"
 #include "profile-widget/profilewidget2.h"
 #include "desktop-widgets/undocommands.h"
+#include "core/qthelper.h"
 
 class MinMaxAvgWidgetPrivate {
 public:
@@ -128,6 +130,12 @@ void MinMaxAvgWidget::overrideMaxToolTipText(const QString &newTip)
 	d->maxValue->setToolTip(newTip);
 }
 
+void MinMaxAvgWidget::setAvgVisibility(const bool visible)
+{
+	d->avgIco->setVisible(visible);
+	d->avgValue->setVisible(visible);
+}
+
 RenumberDialog *RenumberDialog::instance()
 {
 	static RenumberDialog *self = new RenumberDialog(MainWindow::instance());
@@ -302,7 +310,7 @@ void ShiftImageTimesDialog::syncCameraClicked()
 	QStringList fileNames = QFileDialog::getOpenFileNames(this,
 							      tr("Open image file"),
 							      DiveListView::lastUsedImageDir(),
-							      tr("Image files (*.jpg *.jpeg *.pnm *.tif *.tiff)"));
+							      tr("Image files (*.jpg *.jpeg)"));
 	if (fileNames.isEmpty())
 		return;
 
@@ -348,8 +356,11 @@ ShiftImageTimesDialog::ShiftImageTimesDialog(QWidget *parent, QStringList fileNa
 	connect(ui.buttonBox, SIGNAL(clicked(QAbstractButton *)), this, SLOT(buttonClicked(QAbstractButton *)));
 	connect(ui.syncCamera, SIGNAL(clicked()), this, SLOT(syncCameraClicked()));
 	connect(ui.timeEdit, SIGNAL(timeChanged(const QTime &)), this, SLOT(timeEditChanged(const QTime &)));
+	connect(ui.backwards, SIGNAL(toggled(bool)), this, SLOT(timeEditChanged()));
 	connect(ui.matchAllImages, SIGNAL(toggled(bool)), this, SLOT(matchAllImagesToggled(bool)));
 	dcImageEpoch = (time_t)0;
+	
+	updateInvalid();
 }
 
 time_t ShiftImageTimesDialog::amount() const
@@ -373,9 +384,16 @@ void ShiftImageTimesDialog::updateInvalid()
 	timestamp_t timestamp;
 	bool allValid = true;
 	ui.warningLabel->hide();
-	ui.invalidLabel->hide();
-	QDateTime time = QDateTime::fromTime_t(displayed_dive.when, Qt::UTC);
-	ui.invalidLabel->setText("Dive:" + time.toString() + "\n");
+	ui.invalidFilesText->hide();
+	QDateTime time_first = QDateTime::fromTime_t(first_selected_dive()->when, Qt::UTC);
+	QDateTime time_last = QDateTime::fromTime_t(last_selected_dive()->when, Qt::UTC);
+	if (first_selected_dive() == last_selected_dive())
+		ui.invalidFilesText->setPlainText(tr("Selected dive date/time") + ": " + time_first.toString());
+	else {
+		ui.invalidFilesText->setPlainText(tr("First selected dive date/time") + ": " + time_first.toString());
+		ui.invalidFilesText->append(tr("Last selected dive date/time") + ": " + time_last.toString());
+	}
+	ui.invalidFilesText->append(tr("\nFiles with inappropriate date/time") + ":");
 
 	Q_FOREACH (const QString &fileName, fileNames) {
 		if (picture_check_valid(fileName.toUtf8().data(), m_amount))
@@ -383,14 +401,17 @@ void ShiftImageTimesDialog::updateInvalid()
 
 		// We've found invalid image
 		timestamp = picture_get_timestamp(fileName.toUtf8().data());
-		time.setTime_t(timestamp + m_amount);
-		ui.invalidLabel->setText(ui.invalidLabel->text() + fileName + " " + time.toString() + "\n");
+		time_first.setTime_t(timestamp + m_amount);
+		if (timestamp == 0)
+			ui.invalidFilesText->append(fileName + " - " + tr("No Exif date/time found"));
+		else
+			ui.invalidFilesText->append(fileName + " - " + time_first.toString());
 		allValid = false;
 	}
 
 	if (!allValid){
 		ui.warningLabel->show();
-		ui.invalidLabel->show();
+		ui.invalidFilesText->show();
 	}
 }
 
@@ -402,6 +423,13 @@ void ShiftImageTimesDialog::timeEditChanged(const QTime &time)
 	updateInvalid();
 }
 
+void ShiftImageTimesDialog::timeEditChanged()
+{
+	if ((m_amount > 0) == ui.backwards->isChecked())
+			m_amount *= -1;
+	if (m_amount) updateInvalid();
+}
+
 URLDialog::URLDialog(QWidget *parent) : QDialog(parent)
 {
 	ui.setupUi(this);
