@@ -1,16 +1,18 @@
+// SPDX-License-Identifier: GPL-2.0
 #include "desktop-widgets/subsurfacewebservices.h"
 #include "core/helpers.h"
 #include "core/webservice.h"
 #include "desktop-widgets/mainwindow.h"
 #include "desktop-widgets/usersurvey.h"
 #include "core/divelist.h"
-#include "desktop-widgets/globe.h"
+#include "desktop-widgets/mapwidget.h"
 #include "desktop-widgets/tab-widgets/maintab.h"
 #include "core/display.h"
 #include "core/membuffer.h"
 #include "core/subsurface-qt/SettingsObjectWrapper.h"
 #include <errno.h>
 #include "core/cloudstorage.h"
+#include "core/dive.h"
 
 #include <QDir>
 #include <QHttpMultiPart>
@@ -69,7 +71,7 @@ static bool merge_locations_into_dives(void)
 						qDebug() << "processing gpsfix @" << get_dive_date_string(gpsfix->when) <<
 							    "which is withing six hours of dive from" <<
 							    get_dive_date_string(dive->when) << "until" <<
-							    get_dive_date_string(dive->when + dive->duration.seconds);
+							    get_dive_date_string(dive_endtime(dive));
 					/*
 					 * If position is fixed during dive. This is the good one.
 					 * Asign and mark position, and end gps_location loop
@@ -105,7 +107,7 @@ static bool merge_locations_into_dives(void)
 								if (verbose)
 									qDebug() << "which is closer to the start of the dive, do continue with that";
 								continue;
-							} else if (gpsfix->when > dive->when + dive->duration.seconds) {
+							} else if (gpsfix->when > dive_endtime(dive)) {
 								if (verbose)
 									qDebug() << "which is even later after the end of the dive, so pick the previous one";
 								copy_gps_location(gpsfix, dive);
@@ -114,7 +116,7 @@ static bool merge_locations_into_dives(void)
 								break;
 							} else {
 								/* ok, gpsfix is before, nextgpsfix is after */
-								if (dive->when - gpsfix->when <= nextgpsfix->when - (dive->when + dive->duration.seconds)) {
+								if (dive->when - gpsfix->when <= nextgpsfix->when - dive_endtime(dive)) {
 									if (verbose)
 										qDebug() << "pick the one before as it's closer to the start";
 									copy_gps_location(gpsfix, dive);
@@ -146,7 +148,7 @@ static bool merge_locations_into_dives(void)
 					/* If position is out of SAME_GROUP range and in the future, mark position for
 					 * next dive iteration and end the gps_location loop
 					 */
-					if (gpsfix->when >= dive->when + dive->duration.seconds + SAME_GROUP) {
+					if (gpsfix->when >= dive_endtime(dive) + SAME_GROUP) {
 						tracer = j;
 						break;
 					}
@@ -397,7 +399,8 @@ SubsurfaceWebServices::SubsurfaceWebServices(QWidget *parent, Qt::WindowFlags f)
 
 	if (userid.isEmpty() &&
 	    !same_string(prefs.cloud_storage_email, "") &&
-	    !same_string(prefs.cloud_storage_password, ""))
+	    !same_string(prefs.cloud_storage_password, "") &&
+	    GpsLocation::hasInstance())
 		userid = GpsLocation::instance()->getUserid(prefs.cloud_storage_email, prefs.cloud_storage_password);
 
 	ui.userID->setText(userid);
@@ -448,7 +451,7 @@ void SubsurfaceWebServices::buttonClicked(QAbstractButton *button)
 		bool qSaveUid = ui.saveUidLocal->checkState();
 		SettingsObjectWrapper::instance()->cloud_storage->setSaveUserIdLocal(qSaveUid);
 
-                //WARN: Dirk, this seems to be wrong, I coundn't really understand the code.
+		//WARN: Dirk, this seems to be wrong, I coundn't really understand the code.
 		if (qSaveUid) {
 			QString qSettingUid = s.value("subsurface_webservice_uid").toString();
 			QString qFileUid = QString(prefs.userid);
@@ -476,14 +479,12 @@ void SubsurfaceWebServices::buttonClicked(QAbstractButton *button)
 				i--; // otherwise we skip one site
 			}
 		}
-#ifndef NO_MARBLE
 		// finally now that all the extra GPS fixes that weren't used have been deleted
-		// we can update the globe
+		// we can update the map
 		if (changed) {
-			GlobeGPS::instance()->repopulateLabels();
-			GlobeGPS::instance()->centerOnDiveSite(get_dive_site_by_uuid(current_dive->dive_site_uuid));
+			MapWidget::instance()->repopulateLabels();
+			MapWidget::instance()->centerOnDiveSite(get_dive_site_by_uuid(current_dive->dive_site_uuid));
 		}
-#endif
 
 	} break;
 	case QDialogButtonBox::RejectRole:
