@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 /*
  * maintab.cpp
  *
@@ -6,7 +7,7 @@
  */
 #include "desktop-widgets/tab-widgets/maintab.h"
 #include "desktop-widgets/mainwindow.h"
-#include "desktop-widgets/globe.h"
+#include "desktop-widgets/mapwidget.h"
 #include "core/helpers.h"
 #include "core/statistics.h"
 #include "desktop-widgets/modeldelegates.h"
@@ -70,10 +71,8 @@ MainTab::MainTab(QWidget *parent) : QTabWidget(parent),
 	closeMessage();
 
 	connect(ui.editDiveSiteButton, SIGNAL(clicked()), MainWindow::instance(), SIGNAL(startDiveSiteEdit()));
-#ifndef NO_MARBLE
-	connect(ui.location, &DiveLocationLineEdit::entered, GlobeGPS::instance(), &GlobeGPS::centerOnIndex);
-	connect(ui.location, &DiveLocationLineEdit::currentChanged, GlobeGPS::instance(), &GlobeGPS::centerOnIndex);
-#endif
+	connect(ui.location, &DiveLocationLineEdit::entered, MapWidget::instance(), &MapWidget::centerOnIndex);
+	connect(ui.location, &DiveLocationLineEdit::currentChanged, MapWidget::instance(), &MapWidget::centerOnIndex);
 
 	QAction *action = new QAction(tr("Apply changes"), this);
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(acceptChanges()));
@@ -459,7 +458,7 @@ void MainTab::updateDiveInfo(bool clear)
 
 		if (ds) {
 			ui.location->setCurrentDiveSiteUuid(ds->uuid);
-			ui.locationTags->setText(constructLocationTags(ds->uuid));
+			ui.locationTags->setText(constructLocationTags(ds, true));
 		} else {
 			ui.location->clear();
 			clear_dive_site(&displayed_dive_site);
@@ -590,6 +589,7 @@ void MainTab::updateDiveInfo(bool clear)
 			ui.locationTags->hide();
 		else
 			ui.locationTags->show();
+		ui.editDiveSiteButton->setEnabled(!ui.location->text().isEmpty());
 		/* unset the special value text for date and time, just in case someone dove at midnight */
 		ui.dateEdit->setSpecialValueText(QString(""));
 		ui.timeEdit->setSpecialValueText(QString(""));
@@ -776,6 +776,8 @@ void MainTab::acceptChanges()
 		MainWindow::instance()->dive_list()->unselectDives();
 		selected_dive = get_divenr(added_dive);
 		amount_selected = 1;
+		// finally, make sure we get the tags
+		saveTags();
 	} else if (MainWindow::instance() && MainWindow::instance()->dive_list()->selectedTrips().count() == 1) {
 		/* now figure out if things have changed */
 		if (displayedTrip.notes && !same_string(displayedTrip.notes, currentTrip->notes)) {
@@ -796,7 +798,6 @@ void MainTab::acceptChanges()
 			addedId = displayed_dive.id;
 		}
 		struct dive *cd = current_dive;
-		struct divecomputer *displayed_dc = get_dive_dc(&displayed_dive, dc_number);
 		// now check if something has changed and if yes, edit the selected dives that
 		// were identical with the master dive shown (and mark the divelist as changed)
 		if (!same_string(displayed_dive.suit, cd->suit))
@@ -904,7 +905,7 @@ void MainTab::acceptChanges()
 				qDebug() << "delete now unused dive site" << ((ds && ds->name) ? ds->name : "without name");
 			}
 			delete_dive_site(oldUuid);
-			GlobeGPS::instance()->reload();
+			MapWidget::instance()->reload();
 		}
 		// the code above can change the correct uuid for the displayed dive site - and the
 		// code below triggers an update of the display without re-initializing displayed_dive
@@ -968,7 +969,7 @@ void MainTab::acceptChanges()
 	weightModel->changed = false;
 	MainWindow::instance()->setEnabledToolbar(true);
 	acceptingEdit = false;
-	ui.editDiveSiteButton->setEnabled(true);
+	ui.editDiveSiteButton->setEnabled(!ui.location->text().isEmpty());
 }
 
 void MainTab::resetPallete()
@@ -1036,9 +1037,7 @@ void MainTab::rejectChanges()
 	}
 	// the user could have edited the location and then canceled the edit
 	// let's get the correct location back in view
-#ifndef NO_MARBLE
-	GlobeGPS::instance()->centerOnDiveSite(get_dive_site_by_uuid(displayed_dive.dive_site_uuid));
-#endif
+	MapWidget::instance()->centerOnDiveSite(get_dive_site_by_uuid(displayed_dive.dive_site_uuid));
 	// show the profile and dive info
 	MainWindow::instance()->graphics()->replot();
 	MainWindow::instance()->setEnabledToolbar(true);
@@ -1046,7 +1045,7 @@ void MainTab::rejectChanges()
 	weightModel->changed = false;
 	cylindersModel->updateDive();
 	weightModel->updateDive();
-	ui.editDiveSiteButton->setEnabled(true);
+	ui.editDiveSiteButton->setEnabled(!ui.location->text().isEmpty());
 }
 #undef EDIT_TEXT2
 
@@ -1145,7 +1144,6 @@ void MainTab::divetype_Changed(int index)
 {
 	if (editMode == IGNORE)
 		return;
-	struct divecomputer *displayed_dc = get_dive_dc(&displayed_dive, dc_number);
 	displayed_dc->divemode = (enum dive_comp_type) index;
 	update_setpoint_events(&displayed_dive, displayed_dc);
 	markChangedWidget(ui.DiveType);
