@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #ifndef QMLMANAGER_H
 #define QMLMANAGER_H
 
@@ -6,13 +7,18 @@
 #include <QNetworkAccessManager>
 #include <QScreen>
 #include <QElapsedTimer>
+#include <QColor>
 
+#if defined(BT_SUPPORT)
+#include "core/btdiscovery.h"
+#endif
 #include "core/gpslocation.h"
+#include "core/downloadfromdcthread.h"
 #include "qt-models/divelistmodel.h"
 
 class QMLManager : public QObject {
 	Q_OBJECT
-	Q_ENUMS(credentialStatus_t)
+	Q_ENUMS(cloud_status_qml)
 	Q_PROPERTY(QString cloudUserName READ cloudUserName WRITE setCloudUserName NOTIFY cloudUserNameChanged)
 	Q_PROPERTY(QString cloudPassword READ cloudPassword WRITE setCloudPassword NOTIFY cloudPasswordChanged)
 	Q_PROPERTY(QString cloudPin READ cloudPin WRITE setCloudPin NOTIFY cloudPinChanged)
@@ -21,12 +27,13 @@ class QMLManager : public QObject {
 	Q_PROPERTY(bool locationServiceAvailable READ locationServiceAvailable WRITE setLocationServiceAvailable NOTIFY locationServiceAvailableChanged)
 	Q_PROPERTY(int distanceThreshold READ distanceThreshold WRITE setDistanceThreshold NOTIFY distanceThresholdChanged)
 	Q_PROPERTY(int timeThreshold READ timeThreshold WRITE setTimeThreshold NOTIFY timeThresholdChanged)
+	Q_PROPERTY(QString theme READ theme WRITE setTheme NOTIFY themeChanged)
 	Q_PROPERTY(bool loadFromCloud READ loadFromCloud WRITE setLoadFromCloud NOTIFY loadFromCloudChanged)
 	Q_PROPERTY(QString startPageText READ startPageText WRITE setStartPageText NOTIFY startPageTextChanged)
 	Q_PROPERTY(bool verboseEnabled READ verboseEnabled WRITE setVerboseEnabled NOTIFY verboseEnabledChanged)
-	Q_PROPERTY(credentialStatus_t credentialStatus READ credentialStatus WRITE setCredentialStatus NOTIFY credentialStatusChanged)
-	Q_PROPERTY(credentialStatus_t oldStatus READ oldStatus WRITE setOldStatus NOTIFY oldStatusChanged)
-	Q_PROPERTY(int accessingCloud READ accessingCloud WRITE setAccessingCloud NOTIFY accessingCloudChanged)
+	Q_PROPERTY(cloud_status_qml credentialStatus READ credentialStatus WRITE setCredentialStatus NOTIFY credentialStatusChanged)
+	Q_PROPERTY(cloud_status_qml oldStatus READ oldStatus WRITE setOldStatus NOTIFY oldStatusChanged)
+	Q_PROPERTY(QString notificationText READ notificationText WRITE setNotificationText NOTIFY notificationTextChanged)
 	Q_PROPERTY(bool syncToCloud READ syncToCloud WRITE setSyncToCloud NOTIFY syncToCloudChanged)
 	Q_PROPERTY(int updateSelectedDive READ updateSelectedDive WRITE setUpdateSelectedDive NOTIFY updateSelectedDiveChanged)
 	Q_PROPERTY(int selectedDiveTimestamp READ selectedDiveTimestamp WRITE setSelectedDiveTimestamp NOTIFY selectedDiveTimestampChanged)
@@ -35,18 +42,21 @@ class QMLManager : public QObject {
 	Q_PROPERTY(QStringList divemasterInit READ divemasterInit CONSTANT)
 	Q_PROPERTY(QStringList cylinderInit READ cylinderInit CONSTANT)
 	Q_PROPERTY(bool showPin READ showPin WRITE setShowPin NOTIFY showPinChanged)
+	Q_PROPERTY(QString progressMessage READ progressMessage WRITE setProgressMessage NOTIFY progressMessageChanged)
+	Q_PROPERTY(bool libdcLog READ libdcLog WRITE setLibdcLog NOTIFY libdcLogChanged)
+	Q_PROPERTY(bool developer READ developer WRITE setDeveloper NOTIFY developerChanged)
+	Q_PROPERTY(bool btEnabled READ btEnabled WRITE setBtEnabled NOTIFY btEnabledChanged)
 
 public:
 	QMLManager();
 	~QMLManager();
 
-	enum credentialStatus_t {
-		INCOMPLETE,
-		UNKNOWN,
-		INVALID,
-		VALID_EMAIL,
-		VALID,
-		NOCLOUD
+	enum cloud_status_qml {
+		CS_UNKNOWN,
+		CS_INCORRECT_USER_PASSWD,
+		CS_NEED_TO_VERIFY,
+		CS_VERIFIED,
+		CS_NOCLOUD
 	};
 
 	static QMLManager *instance();
@@ -75,6 +85,9 @@ public:
 	int timeThreshold() const;
 	void setTimeThreshold(int time);
 
+	QString theme() const;
+	void setTheme(QString theme);
+
 	bool loadFromCloud() const;
 	void setLoadFromCloud(bool done);
 	void syncLoadFromCloud();
@@ -82,17 +95,17 @@ public:
 	QString startPageText() const;
 	void setStartPageText(const QString& text);
 
-	credentialStatus_t credentialStatus() const;
-	void setCredentialStatus(const credentialStatus_t value);
+	cloud_status_qml credentialStatus() const;
+	void setCredentialStatus(const cloud_status_qml value);
 
-	credentialStatus_t oldStatus() const;
-	void setOldStatus(const credentialStatus_t value);
+	cloud_status_qml oldStatus() const;
+	void setOldStatus(const cloud_status_qml value);
 
 	QString logText() const;
 	void setLogText(const QString &logText);
 
-	int accessingCloud() const;
-	void setAccessingCloud(int status);
+	QString notificationText() const;
+	void setNotificationText(QString text);
 
 	bool syncToCloud() const;
 	void setSyncToCloud(bool status);
@@ -103,6 +116,18 @@ public:
 	int selectedDiveTimestamp() const;
 	void setSelectedDiveTimestamp(int when);
 
+	QString progressMessage() const;
+	void setProgressMessage(QString text);
+
+	bool libdcLog() const;
+	void setLibdcLog(bool value);
+
+	bool developer() const;
+	void setDeveloper(bool value);
+
+	bool btEnabled() const;
+	void setBtEnabled(bool value);
+
 	typedef void (QMLManager::*execute_function_type)();
 	DiveListSortModel *dlSortModel;
 
@@ -112,6 +137,14 @@ public:
 	QStringList cylinderInit() const;
 	bool showPin() const;
 	void setShowPin(bool enable);
+	Q_INVOKABLE void setStatusbarColor(QColor color);
+#if defined(BT_SUPPORT)
+	void btHostModeChange(QBluetoothLocalDevice::HostMode state);
+#endif
+
+#if defined(Q_OS_ANDROID)
+	void writeToAppLogFile(QString logText);
+#endif
 
 public slots:
 	void applicationStateChanged(Qt::ApplicationState state);
@@ -123,14 +156,14 @@ public slots:
 	void handleSslErrors(const QList<QSslError> &errors);
 	void retrieveUserid();
 	void loadDivesWithValidCredentials();
-	void loadDiveProgress(int percent);
 	void provideAuth(QNetworkReply *reply, QAuthenticator *auth);
 	void commitChanges(QString diveId, QString date, QString location, QString gps,
 			   QString duration, QString depth, QString airtemp,
 			   QString watertemp, QString suit, QString buddy,
 			   QString diveMaster, QString weight, QString notes, QString startpressure,
-			   QString endpressure, QString gasmix, QString cylinder);
+			   QString endpressure, QString gasmix, QString cylinder, int rating, int visibility);
 	void changesNeedSaving();
+	void openNoCloudRepo();
 	void saveChangesLocal();
 	void saveChangesCloud(bool forceRemoteSync);
 	void deleteDive(int id);
@@ -141,7 +174,10 @@ public slots:
 	void sendGpsData();
 	void downloadGpsData();
 	void populateGpsData();
+	void cancelDownloadDC();
 	void clearGpsData();
+	void clearCredentials();
+	void cancelCredentialsPinSetup();
 	void finishSetup();
 	void openLocalThenRemote(QString url);
 	void mergeLocalRepo();
@@ -179,12 +215,12 @@ private:
 	QNetworkRequest request;
 	struct dive *deletedDive;
 	struct dive_trip *deletedTrip;
-	int m_accessingCloud;
+	QString m_notificationText;
 	bool m_syncToCloud;
 	int m_updateSelectedDive;
 	int m_selectedDiveTimestamp;
-	credentialStatus_t m_credentialStatus;
-	credentialStatus_t m_oldStatus;
+	cloud_status_qml m_credentialStatus;
+	cloud_status_qml m_oldStatus;
 	qreal m_lastDevicePixelRatio;
 	QElapsedTimer timer;
 	bool alreadySaving;
@@ -194,6 +230,17 @@ private:
 	bool checkDepth(DiveObjectHelper *myDive, struct dive *d, QString depth);
 	bool currentGitLocalOnly;
 	bool m_showPin;
+	DCDeviceData *m_device_data;
+	QString m_progressMessage;
+	bool m_libdcLog;
+	bool m_developer;
+	bool m_btEnabled;
+
+#if defined(Q_OS_ANDROID)
+	QString appLogFileName;
+	QFile appLogFile;
+	bool appLogFileOpen;
+#endif
 
 signals:
 	void cloudUserNameChanged();
@@ -204,17 +251,22 @@ signals:
 	void verboseEnabledChanged();
 	void logTextChanged();
 	void timeThresholdChanged();
+	void themeChanged();
 	void distanceThresholdChanged();
 	void loadFromCloudChanged();
 	void startPageTextChanged();
 	void credentialStatusChanged();
 	void oldStatusChanged();
-	void accessingCloudChanged();
+	void notificationTextChanged();
 	void syncToCloudChanged();
 	void updateSelectedDiveChanged();
 	void selectedDiveTimestampChanged();
 	void showPinChanged();
 	void sendScreenChanged(QScreen *screen);
+	void progressMessageChanged();
+	void libdcLogChanged();
+	void developerChanged();
+	void btEnabledChanged();
 };
 
 #endif
