@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #ifndef PROFILEWIDGET2_H
 #define PROFILEWIDGET2_H
 
@@ -21,7 +22,6 @@ class RulerItem2;
 struct dive;
 struct plot_info;
 class ToolTipItem;
-class DiveMeanDepth;
 class DiveReportedCeiling;
 class DiveTextItem;
 class TemperatureAxis;
@@ -35,7 +35,6 @@ class DiveProfileItem;
 class TimeAxis;
 class DiveTemperatureItem;
 class DiveHeartrateItem;
-class PercentageItem;
 class DiveGasPressureItem;
 class DiveCalculatedCeiling;
 class DiveCalculatedTissue;
