@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #include "cylindermodel.h"
 #include "tankinfomodel.h"
 #include "models.h"
@@ -134,12 +135,10 @@ QVariant CylindersModel::data(const QModelIndex &index, int role) const
 
 	if (!index.isValid() || index.row() >= MAX_CYLINDERS)
 		return ret;
-	
-	int mapping[MAX_CYLINDERS];
+
 	int same_gas = -1;
 	cylinder_t *cyl = &displayed_dive.cylinder[index.row()];
-	struct gasmix *mygas = &cyl->gasmix;
-	
+
 	switch (role) {
 	case Qt::BackgroundRole: {
 		switch (index.column()) {
@@ -232,16 +231,9 @@ QVariant CylindersModel::data(const QModelIndex &index, int role) const
 	case Qt::DecorationRole:
 	case Qt::SizeHintRole:
 		if (index.column() == REMOVE) {
-			same_gas = -1;
-			for (int i = 0; i < MAX_CYLINDERS; i++) {
-				mapping[i] = i;
-				if (i == index.row() || cylinder_none(&displayed_dive.cylinder[i]))
-					continue;
-				struct gasmix *gas2 = &displayed_dive.cylinder[i].gasmix;
-				if (gasmix_distance(mygas, gas2) == 0 && is_cylinder_used(&displayed_dive, i))
-					same_gas = i;
-			}
-			if (same_gas == -1 &&
+			same_gas = same_gasmix_cylinder(cyl, index.row(), &displayed_dive, false);
+
+			if ((in_planner() || same_gas == -1) &&
 				((DivePlannerPointsModel::instance()->currentMode() != DivePlannerPointsModel::NOTHING &&
 				DivePlannerPointsModel::instance()->tankInUse(index.row())) ||
 				(DivePlannerPointsModel::instance()->currentMode() == DivePlannerPointsModel::NOTHING &&
@@ -255,16 +247,9 @@ QVariant CylindersModel::data(const QModelIndex &index, int role) const
 	case Qt::ToolTipRole:
 		switch (index.column()) {
 		case REMOVE:
-			same_gas = -1;
-			for (int i = 0; i < MAX_CYLINDERS; i++) {
-				mapping[i] = i;
-				if (i == index.row() || cylinder_none(&displayed_dive.cylinder[i]))
-					continue;
-				struct gasmix *gas2 = &displayed_dive.cylinder[i].gasmix;
-				if (gasmix_distance(mygas, gas2) == 0 && is_cylinder_used(&displayed_dive, i))
-					same_gas = i;
-			}
-			if (same_gas == -1 &&
+			same_gas = same_gasmix_cylinder(cyl, index.row(), &displayed_dive, false);
+
+			if ((in_planner() || same_gas == -1) &&
 				((DivePlannerPointsModel::instance()->currentMode() != DivePlannerPointsModel::NOTHING &&
 				DivePlannerPointsModel::instance()->tankInUse(index.row())) ||
 				(DivePlannerPointsModel::instance()->currentMode() == DivePlannerPointsModel::NOTHING &&
@@ -473,6 +458,7 @@ void CylindersModel::add()
 	rows++;
 	changed = true;
 	endInsertRows();
+	emit dataChanged(createIndex(row, 0), createIndex(row, COLUMNS - 1));
 }
 
 void CylindersModel::clear()
@@ -483,15 +469,33 @@ void CylindersModel::clear()
 	}
 }
 
+static bool show_cylinder(struct dive *dive, int i)
+{
+	cylinder_t *cyl = dive->cylinder + i;
+
+	if (is_cylinder_used(dive, i))
+		return true;
+	if (cylinder_none(cyl))
+		return false;
+	if (cyl->start.mbar || cyl->sample_start.mbar ||
+	    cyl->end.mbar || cyl->sample_end.mbar)
+		return true;
+	if (cyl->manually_added)
+		return true;
+
+	/*
+	 * The cylinder has some data, but none of it is very interesting,
+	 * it has no pressures and no gas switches. Do we want to show it?
+	 */
+	return prefs.display_unused_tanks;
+}
+
 void CylindersModel::updateDive()
 {
 	clear();
 	rows = 0;
 	for (int i = 0; i < MAX_CYLINDERS; i++) {
-		if (!cylinder_none(&displayed_dive.cylinder[i]) &&
-				(prefs.display_unused_tanks ||
-				 is_cylinder_used(&displayed_dive, i) ||
-				 displayed_dive.cylinder[i].manually_added))
+		if (show_cylinder(&displayed_dive, i))
 			rows = i + 1;
 	}
 	if (rows > 0) {
@@ -506,10 +510,8 @@ void CylindersModel::copyFromDive(dive *d)
 		return;
 	rows = 0;
 	for (int i = 0; i < MAX_CYLINDERS; i++) {
-		if (!cylinder_none(&d->cylinder[i]) &&
-				(is_cylinder_used(d, i) || prefs.display_unused_tanks)) {
+		if (show_cylinder(d, i))
 			rows = i + 1;
-		}
 	}
 	if (rows > 0) {
 		beginInsertRows(QModelIndex(), 0, rows - 1);
@@ -541,18 +543,10 @@ void CylindersModel::remove(const QModelIndex &index)
 	if (index.column() != REMOVE) {
 		return;
 	}
-	int same_gas = -1;
 	cylinder_t *cyl = &displayed_dive.cylinder[index.row()];
-	struct gasmix *mygas = &cyl->gasmix;
-	for (int i = 0; i < MAX_CYLINDERS; i++) {
-		mapping[i] = i;
-		if (i == index.row() || cylinder_none(&displayed_dive.cylinder[i]))
-			continue;
-		struct gasmix *gas2 = &displayed_dive.cylinder[i].gasmix;
-		if (gasmix_distance(mygas, gas2) == 0 && is_cylinder_used(&displayed_dive, i))
-			same_gas = i;
-	}
-	if (same_gas == -1 &&
+	int same_gas = same_gasmix_cylinder(cyl, index.row(), &displayed_dive, false);
+
+	if ((in_planner() || same_gas == -1) &&
 			((DivePlannerPointsModel::instance()->currentMode() != DivePlannerPointsModel::NOTHING &&
 				DivePlannerPointsModel::instance()->tankInUse(index.row())) ||
 			(DivePlannerPointsModel::instance()->currentMode() == DivePlannerPointsModel::NOTHING &&
@@ -569,6 +563,8 @@ void CylindersModel::remove(const QModelIndex &index)
 		// as first gas
 		memmove(cyl, &displayed_dive.cylinder[same_gas], sizeof(*cyl));
 		remove_cylinder(&displayed_dive, same_gas);
+		for (int i = 0; i < same_gas - 1; i++)
+			mapping[i] = i;
 		mapping[same_gas] = 0;
 		for (int i = same_gas + 1; i < MAX_CYLINDERS; i++)
 			mapping[i] = i - 1;
@@ -576,17 +572,40 @@ void CylindersModel::remove(const QModelIndex &index)
 		remove_cylinder(&displayed_dive, index.row());
 		if (same_gas > index.row())
 			same_gas--;
+		for (int i = 0; i < index.row(); i++)
+			mapping[i] = i;
 		mapping[index.row()] = same_gas;
 		for (int i = index.row() + 1; i < MAX_CYLINDERS; i++)
 			mapping[i] = i - 1;
 	}
+	cylinder_renumber(&displayed_dive, mapping);
+	if (in_planner())
+		DivePlannerPointsModel::instance()->cylinderRenumber(mapping);
 	changed = true;
 	endRemoveRows();
-	struct divecomputer *dc = &displayed_dive.dc;
-	while (dc) {
-		dc_cylinder_renumber(&displayed_dive, dc, mapping);
-		dc = dc->next;
+	dataChanged(index, index);
+}
+
+void CylindersModel::moveAtFirst(int cylid)
+{
+	int mapping[MAX_CYLINDERS];
+	cylinder_t temp_cyl;
+
+	beginMoveRows(QModelIndex(), cylid, cylid, QModelIndex(), 0);
+	memmove(&temp_cyl, &displayed_dive.cylinder[cylid], sizeof(temp_cyl));
+	for (int i = cylid - 1; i >= 0; i--) {
+		memmove(&displayed_dive.cylinder[i + 1], &displayed_dive.cylinder[i], sizeof(temp_cyl));
+		mapping[i] = i + 1;
 	}
+	memmove(&displayed_dive.cylinder[0], &temp_cyl, sizeof(temp_cyl));
+	mapping[cylid] = 0;
+	for (int i = cylid + 1; i < MAX_CYLINDERS; i++)
+		mapping[i] = i;
+	cylinder_renumber(&displayed_dive, mapping);
+	if (in_planner())
+		DivePlannerPointsModel::instance()->cylinderRenumber(mapping);
+	changed = true;
+	endMoveRows();
 }
 
 void CylindersModel::updateDecoDepths(pressure_t olddecopo2)
