@@ -32,6 +32,8 @@ if [ "$NOPULL" = "" ] ; then
 	pushd kirigami
 	git checkout master
 	git pull origin master
+	# if we want to pin a specific Kirigami version, we can do this here
+	git checkout 6a16e00a692914b1d3918ba57e658342de4d2c23
 	popd
 fi
 if [ ! -d breeze-icons ] ; then
@@ -58,16 +60,22 @@ cp $BREEZE/icons/actions/24/document-edit.svg $MC/icons
 cp $BREEZE/icons/actions/24/document-save.svg $MC/icons
 cp $BREEZE/icons/actions/24/go-next.svg $MC/icons
 cp $BREEZE/icons/actions/24/go-previous.svg $MC/icons
+cp $BREEZE/icons/actions/24/go-up.svg $MC/icons
 cp $BREEZE/icons/actions/16/view-readermode.svg $MC/icons
 cp $BREEZE/icons/actions/24/application-menu.svg $MC/icons
 cp $BREEZE/icons/actions/22/gps.svg $MC/icons
 cp $BREEZE/icons/actions/24/trash-empty.svg $MC/icons
 cp $BREEZE/icons/actions/24/list-add.svg $MC/icons
 cp $BREEZE/icons/actions/22/handle-left.svg $MC/icons
+cp $BREEZE/icons/actions/22/handle-right.svg $MC/icons
+cp $BREEZE/icons/actions/22/overflow-menu.svg $MC/icons
 
 # kirigami now needs the breeze-icons internally as well
 pushd $MC
 ln -s $SRC/$BREEZE .
+sed -i -e "s/visible: root.action/visible: root.action \&\& \!Qt.inputMethod.visible/g" src/controls/private/ActionButton.qml
+sed -i -e "s/visible: root.leftAction/visible: root.leftAction \&\& \!Qt.inputMethod.visible/g" src/controls/private/ActionButton.qml
+sed -i -e "s/visible: root.rightAction/visible: root.rightAction \&\& \!Qt.inputMethod.visible/g" src/controls/private/ActionButton.qml
 popd
 
 echo org.kde.plasma.kirigami synced from upstream
