@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 /* main.c */
 #include <locale.h>
 #include <stdio.h>
@@ -15,6 +16,7 @@
 #include "desktop-widgets/diveplanner.h"
 #include "core/color.h"
 #include "core/qthelper.h"
+#include "core/downloadfromdcthread.h" // for fill_computer_list
 
 #include <QStringList>
 #include <QApplication>
@@ -80,10 +82,10 @@ int main(int argc, char **argv)
 	 * the constant numbers we used to get before.
 	 */
 	qsrand(time(NULL));
-
 	setup_system_prefs();
 	copy_prefs(&default_prefs, &prefs);
 	fill_profile_color();
+	fill_computer_list();
 	parse_xml_init();
 	taglist_init_global();
 	init_ui();
@@ -105,8 +107,10 @@ int main(int argc, char **argv)
 	// in case something has gone wrong make sure we show the error message
 	m->showError();
 
-	if (verbose > 0)
+	if (verbose > 0) {
 		print_files();
+		printf("%s\n", QStringLiteral("build with Qt Version %1, runtime from Qt Version %2").arg(QT_VERSION_STR).arg(qVersion()).toUtf8().data());
+	}
 	if (!quit)
 		run_ui();
 	exit_ui();
