@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #ifndef TESTPARSE_H
 #define TESTPARSE_H
 
@@ -25,6 +26,9 @@ private slots:
 
 	int parseCSVmanual(int, std::string);
 	void exportCSVDiveDetails();
+	int parseCSVprofile(int, std::string);
+	void exportCSVDiveProfile();
+	void exportUDDF();
 	void testExport();
 
 private:
