@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #include "testpreferences.h"
 
 #include "core/subsurface-qt/SettingsObjectWrapper.h"
@@ -131,8 +132,6 @@ void TestPreferences::testPreferences()
 	TEST(tecDetails->zoomedPlot(), true);
 	tecDetails->setShowSac(true);
 	TEST(tecDetails->showSac(), true);
-	tecDetails->setGfLowAtMaxDepth(true);
-	TEST(tecDetails->gfLowAtMaxDepth(), true);
 	tecDetails->setDisplayUnusedTanks(true);
 	TEST(tecDetails->displayUnusedTanks(), true);
 	tecDetails->setShowAverageDepth(true);
@@ -174,8 +173,6 @@ void TestPreferences::testPreferences()
 	TEST(tecDetails->zoomedPlot(), false);
 	tecDetails->setShowSac(false);
 	TEST(tecDetails->showSac(), false);
-	tecDetails->setGfLowAtMaxDepth(false);
-	TEST(tecDetails->gfLowAtMaxDepth(), false);
 	tecDetails->setDisplayUnusedTanks(false);
 	TEST(tecDetails->displayUnusedTanks(), false);
 	tecDetails->setShowAverageDepth(false);
@@ -552,7 +549,7 @@ void TestPreferences::testPreferences()
 	TEST(update->lastVersionUsed(), QStringLiteral("tomaz-1"));
 	TEST(update->nextCheck(), date);
 
-	date.addDays(3);
+	date = date.addDays(3);
 	update->setDontCheckForUpdates(false);
 	update->setLastVersionUsed("tomaz-2");
 	update->setNextCheck(date);
